/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

                Copyright (c) 2003-2019, Cauldron Development LLC
                   Copyright (c) 2003-2017, Stanford University
                               All rights reserved.

         The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
       as published by the Free Software Foundation, either version 2.1 of
               the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                 Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
                 License along with the C! library.  If not, see
                         <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
           You may request written permission by emailing the authors.

                  For information regarding this software email:
                                 Joseph Coffland
                          joseph@cauldrondevelopment.com

\******************************************************************************/

#include "PCIVendor.h"

#include <cbang/String.h>

using namespace std;
using namespace cb;


PCIVendor::vendors_t PCIVendor::vendors;


const PCIVendor *PCIVendor::find(uint16_t id) {
  if (vendors.empty()) load();
  vendors_t::iterator it = vendors.find(PCIVendor(id, ""));
  return it == vendors.end() ? 0 : &*it;
}


void PCIVendor::add(uint16_t id, const char *name) {
  vendors.insert(PCIVendor(id, name));
}


void PCIVendor::load() {
  add(0x0010, "Allied Telesis, Inc (Wrong ID)");
  add(0x001c, "PEAK-System Technik GmbH");
  add(0x0033, "Paradyne Corp.");
  add(0x003d, "Lockheed Martin-Marietta Corp");
  add(0x0059, "Tiger Jet Network Inc. (Wrong ID)");
  add(0x0070, "Hauppauge computer works Inc.");
  add(0x0071, "Nebula Electronics Ltd.");
  add(0x0095, "Silicon Image, Inc. (Wrong ID)");
  add(0x00a7, "Teles AG (Wrong ID)");
  add(0x0100, "Ncipher Corp Ltd");
  add(0x0123, "General Dynamics");
  add(0x018a, "LevelOne");
  add(0x021b, "Compaq Computer Corporation");
  add(0x0270, "Hauppauge computer works Inc. (Wrong ID)");
  add(0x02ac, "SpeedStream");
  add(0x0303, "Hewlett-Packard Company (Wrong ID)");
  add(0x0308, "ZyXEL Communications Corporation (Wrong ID)");
  add(0x0315, "SK-Electronics Co., Ltd.");
  add(0x0357, "TTTech Computertechnik AG (Wrong ID)");
  add(0x0402, " Acer aspire one");
  add(0x0432, "SCM Microsystems, Inc.");
  add(0x046d, "Logitech Inc.");
  add(0x0483, "UPEK");
  add(0x04b3, "IBM");
  add(0x04d9, "Filco");
  add(0x04f2, "Chicony Electronics Co.");
  add(0x051d, "APC");
  add(0x058f, "Alcor Micro Corp.");
  add(0x0590, "Omron Corp");
  add(0x05ac, "Apple, Inc.");
  add(0x05e1, "D-MAX");
  add(0x064e, "SUYIN Corporation");
  add(0x0675, "Dynalink");
  add(0x067b, "Prolific Technology Inc.");
  add(0x06fe, "Acresso Software Inc.");
  add(0x0721, "Sapphire, Inc.");
  add(0x0777, "Ubiquiti Networks, Inc.");
  add(0x0795, "Wired Inc.");
  add(0x07d1, "D-Link System Inc");
  add(0x0925, "VIA Technologies, Inc. (Wrong ID)");
  add(0x093a, "KYE Systems Corp.");
  add(0x096e, "USB Rockey dongle from Feitain");
  add(0x0a5c, "Broadcom Corporation");
  add(0x0a89, "BREA Technologies Inc");
  add(0x0a92, "Egosys, Inc.");
  add(0x0ac8, "ASUS");
  add(0x0b05, "Toshiba Bluetooth RFBUS, RFCOM, RFHID");
  add(0x0b0b, "Rhino Equipment Corp.");
  add(0x0c45, "Microdia Ltd.");
  add(0x0cf3, "TP-Link");
  add(0x0d8c, "C-Media Electronics, Inc.");
  add(0x0df6, "Sitecom");
  add(0x0e11, "Compaq Computer Corporation");
  add(0x0e8d, "MediaTek Inc.");
  add(0x0eac, "SHF Communication Technologies AG");
  add(0x0f62, "Acrox Technologies Co., Ltd.");
  add(0x1000, "LSI Logic / Symbios Logic");
  add(0x1001, "Kolter Electronic");
  add(0x1002, "Advanced Micro Devices, Inc. [AMD/ATI]");
  add(0x1003, "ULSI Systems");
  add(0x1004, "VLSI Technology Inc");
  add(0x1005, "Avance Logic Inc. [ALI]");
  add(0x1006, "Reply Group");
  add(0x1007, "NetFrame Systems Inc");
  add(0x1008, "Epson");
  add(0x100a, "Phoenix Technologies");
  add(0x100b, "National Semiconductor Corporation");
  add(0x100c, "Tseng Labs Inc");
  add(0x100d, "AST Research Inc");
  add(0x100e, "Weitek");
  add(0x1010, "Video Logic, Ltd.");
  add(0x1011, "Digital Equipment Corporation");
  add(0x1012, "Micronics Computers Inc");
  add(0x1013, "Cirrus Logic");
  add(0x1014, "IBM");
  add(0x1015, "LSI Logic Corp of Canada");
  add(0x1016, "ICL Personal Systems");
  add(0x1017, "SPEA Software AG");
  add(0x1018, "Unisys Systems");
  add(0x1019, "Elitegroup Computer Systems");
  add(0x101a, "AT&T GIS (NCR)");
  add(0x101b, "Vitesse Semiconductor");
  add(0x101c, "Western Digital");
  add(0x101d, "Maxim Integrated Products");
  add(0x101e, "American Megatrends Inc.");
  add(0x101f, "PictureTel");
  add(0x1020, "Hitachi Computer Products");
  add(0x1021, "OKI Electric Industry Co. Ltd.");
  add(0x1022, "Advanced Micro Devices, Inc. [AMD]");
  add(0x1023, "Trident Microsystems");
  add(0x1024, "Zenith Data Systems");
  add(0x1025, "Acer Incorporated [ALI]");
  add(0x1028, "Dell");
  add(0x1029, "Siemens Nixdorf IS");
  add(0x102a, "LSI Logic");
  add(0x102b, "Matrox Electronics Systems Ltd.");
  add(0x102c, "Chips and Technologies");
  add(0x102d, "Wyse Technology Inc.");
  add(0x102e, "Olivetti Advanced Technology");
  add(0x102f, "Toshiba America");
  add(0x1030, "TMC Research");
  add(0x1031, "Miro Computer Products AG");
  add(0x1032, "Compaq");
  add(0x1033, "NEC Corporation");
  add(0x1034, "Framatome Connectors USA Inc.");
  add(0x1035, "Comp. & Comm. Research Lab");
  add(0x1036, "Future Domain Corp.");
  add(0x1037, "Hitachi Micro Systems");
  add(0x1038, "AMP, Inc");
  add(0x1039, "Silicon Integrated Systems [SiS]");
  add(0x103a, "Seiko Epson Corporation");
  add(0x103b, "Tatung Corp. Of America");
  add(0x103c, "Hewlett-Packard Company");
  add(0x103e, "Solliday Engineering");
  add(0x103f, "Synopsys/Logic Modeling Group");
  add(0x1040, "Accelgraphics Inc.");
  add(0x1041, "Computrend");
  add(0x1042, "Micron");
  add(0x1043, "ASUSTeK Computer Inc.");
  add(0x1044, "Adaptec (formerly DPT)");
  add(0x1045, "OPTi Inc.");
  add(0x1046, "IPC Corporation, Ltd.");
  add(0x1047, "Genoa Systems Corp");
  add(0x1048, "Elsa AG");
  add(0x1049, "Fountain Technologies, Inc.");
  add(0x104a, "STMicroelectronics");
  add(0x104b, "BusLogic");
  add(0x104c, "Texas Instruments");
  add(0x104d, "Sony Corporation");
  add(0x104e, "Oak Technology, Inc");
  add(0x104f, "Co-time Computer Ltd");
  add(0x1050, "Winbond Electronics Corp");
  add(0x1051, "Anigma, Inc.");
  add(0x1052, "?Young Micro Systems");
  add(0x1053, "Young Micro Systems");
  add(0x1054, "Hitachi, Ltd");
  add(0x1055, "Efar Microsystems");
  add(0x1056, "ICL");
  add(0x1057, "Motorola");
  add(0x1058, "Electronics & Telecommunications RSH");
  add(0x1059, "Kontron");
  add(0x105a, "Promise Technology, Inc.");
  add(0x105b, "Foxconn International, Inc.");
  add(0x105c, "Wipro Infotech Limited");
  add(0x105d, "Number 9 Computer Company");
  add(0x105e, "Vtech Computers Ltd");
  add(0x105f, "Infotronic America Inc");
  add(0x1060, "United Microelectronics [UMC]");
  add(0x1061, "I.I.T.");
  add(0x1062, "Maspar Computer Corp");
  add(0x1063, "Ocean Office Automation");
  add(0x1064, "Alcatel");
  add(0x1065, "Texas Microsystems");
  add(0x1066, "PicoPower Technology");
  add(0x1067, "Mitsubishi Electric");
  add(0x1068, "Diversified Technology");
  add(0x1069, "Mylex Corporation");
  add(0x106a, "Aten Research Inc");
  add(0x106b, "Apple Inc.");
  add(0x106c, "Hynix Semiconductor");
  add(0x106d, "Sequent Computer Systems");
  add(0x106e, "DFI, Inc");
  add(0x106f, "City Gate Development Ltd");
  add(0x1070, "Daewoo Telecom Ltd");
  add(0x1071, "Mitac");
  add(0x1072, "GIT Co Ltd");
  add(0x1073, "Yamaha Corporation");
  add(0x1074, "NexGen Microsystems");
  add(0x1075, "Advanced Integrations Research");
  add(0x1076, "Chaintech Computer Co. Ltd");
  add(0x1077, "QLogic Corp.");
  add(0x1078, "Cyrix Corporation");
  add(0x1079, "I-Bus");
  add(0x107a, "NetWorth");
  add(0x107b, "Gateway, Inc.");
  add(0x107c, "LG Electronics [Lucky Goldstar Co. Ltd]");
  add(0x107d, "LeadTek Research Inc.");
  add(0x107e, "Interphase Corporation");
  add(0x107f, "Data Technology Corporation");
  add(0x1080, "Contaq Microsystems");
  add(0x1081, "Supermac Technology");
  add(0x1082, "EFA Corporation of America");
  add(0x1083, "Forex Computer Corporation");
  add(0x1084, "Parador");
  add(0x1085, "Tulip Computers Int'l BV");
  add(0x1086, "J. Bond Computer Systems");
  add(0x1087, "Cache Computer");
  add(0x1088, "Microcomputer Systems (M) Son");
  add(0x1089, "Data General Corporation");
  add(0x108a, "SBS Technologies");
  add(0x108c, "Oakleigh Systems Inc.");
  add(0x108d, "Olicom");
  add(0x108e, "Oracle/SUN");
  add(0x108f, "Systemsoft");
  add(0x1090, "Compro Computer Services, Inc.");
  add(0x1091, "Intergraph Corporation");
  add(0x1092, "Diamond Multimedia Systems");
  add(0x1093, "National Instruments");
  add(0x1094, "First International Computers [FIC]");
  add(0x1095, "Silicon Image, Inc.");
  add(0x1096, "Alacron");
  add(0x1097, "Appian Technology");
  add(0x1098, "Quantum Designs (H.K.) Ltd");
  add(0x1099, "Samsung Electronics Co., Ltd");
  add(0x109a, "Packard Bell");
  add(0x109b, "Gemlight Computer Ltd.");
  add(0x109c, "Megachips Corporation");
  add(0x109d, "Zida Technologies Ltd.");
  add(0x109e, "Brooktree Corporation");
  add(0x109f, "Trigem Computer Inc.");
  add(0x10a0, "Meidensha Corporation");
  add(0x10a1, "Juko Electronics Ind. Co. Ltd");
  add(0x10a2, "Quantum Corporation");
  add(0x10a3, "Everex Systems Inc");
  add(0x10a4, "Globe Manufacturing Sales");
  add(0x10a5, "Smart Link Ltd.");
  add(0x10a6, "Informtech Industrial Ltd.");
  add(0x10a7, "Benchmarq Microelectronics");
  add(0x10a8, "Sierra Semiconductor");
  add(0x10a9, "Silicon Graphics Intl. Corp.");
  add(0x10aa, "ACC Microelectronics");
  add(0x10ab, "Digicom");
  add(0x10ac, "Honeywell IAC");
  add(0x10ad, "Symphony Labs");
  add(0x10ae, "Cornerstone Technology");
  add(0x10af, "Micro Computer Systems Inc");
  add(0x10b0, "CardExpert Technology");
  add(0x10b1, "Cabletron Systems Inc");
  add(0x10b2, "Raytheon Company");
  add(0x10b3, "Databook Inc");
  add(0x10b4, "STB Systems Inc");
  add(0x10b5, "PLX Technology, Inc.");
  add(0x10b6, "Madge Networks");
  add(0x10b7, "3Com Corporation");
  add(0x10b8, "Standard Microsystems Corp [SMC]");
  add(0x10b9, "ULi Electronics Inc.");
  add(0x10ba, "Mitsubishi Electric Corp.");
  add(0x10bb, "Dapha Electronics Corporation");
  add(0x10bc, "Advanced Logic Research");
  add(0x10bd, "Surecom Technology");
  add(0x10be, "Tseng Labs International Co.");
  add(0x10bf, "Most Inc");
  add(0x10c0, "Boca Research Inc.");
  add(0x10c1, "ICM Co., Ltd.");
  add(0x10c2, "Auspex Systems Inc.");
  add(0x10c3, "Samsung Semiconductors, Inc.");
  add(0x10c4, "Award Software International Inc.");
  add(0x10c5, "Xerox Corporation");
  add(0x10c6, "Rambus Inc.");
  add(0x10c7, "Media Vision");
  add(0x10c8, "Neomagic Corporation");
  add(0x10c9, "Dataexpert Corporation");
  add(0x10ca, "Fujitsu Microelectr., Inc.");
  add(0x10cb, "Omron Corporation");
  add(0x10cc, "Mai Logic Incorporated");
  add(0x10cd, "Advanced System Products, Inc");
  add(0x10ce, "Radius");
  add(0x10cf, "Fujitsu Limited.");
  add(0x10d1, "FuturePlus Systems Corp.");
  add(0x10d2, "Molex Incorporated");
  add(0x10d3, "Jabil Circuit Inc");
  add(0x10d4, "Hualon Microelectronics");
  add(0x10d5, "Autologic Inc.");
  add(0x10d6, "Cetia");
  add(0x10d7, "BCM Advanced Research");
  add(0x10d8, "Advanced Peripherals Labs");
  add(0x10d9, "Macronix, Inc. [MXIC]");
  add(0x10da, "Compaq IPG-Austin");
  add(0x10db, "Rohm LSI Systems, Inc.");
  add(0x10dc, "CERN/ECP/EDU");
  add(0x10dd, "Evans & Sutherland");
  add(0x10de, "NVIDIA Corporation");
  add(0x10df, "Emulex Corporation");
  add(0x10e0, "Integrated Micro Solutions Inc.");
  add(0x10e1, "Tekram Technology Co.,Ltd.");
  add(0x10e2, "Aptix Corporation");
  add(0x10e3, "Tundra Semiconductor Corp.");
  add(0x10e4, "Tandem Computers");
  add(0x10e5, "Micro Industries Corporation");
  add(0x10e6, "Gainbery Computer Products Inc.");
  add(0x10e7, "Vadem");
  add(0x10e8, "Applied Micro Circuits Corp.");
  add(0x10e9, "Alps Electric Co., Ltd.");
  add(0x10ea, "Integraphics");
  add(0x10eb, "Artists Graphics");
  add(0x10ec, "Realtek Semiconductor Co., Ltd.");
  add(0x10ed, "Ascii Corporation");
  add(0x10ee, "Xilinx Corporation");
  add(0x10ef, "Racore Computer Products, Inc.");
  add(0x10f0, "Peritek Corporation");
  add(0x10f1, "Tyan Computer");
  add(0x10f2, "Achme Computer, Inc.");
  add(0x10f3, "Alaris, Inc.");
  add(0x10f4, "S-MOS Systems, Inc.");
  add(0x10f5, "NKK Corporation");
  add(0x10f6, "Creative Electronic Systems SA");
  add(0x10f7, "Matsushita Electric Industrial Co., Ltd.");
  add(0x10f8, "Altos India Ltd");
  add(0x10f9, "PC Direct");
  add(0x10fa, "Truevision");
  add(0x10fb, "Thesys Gesellschaft fuer Mikroelektronik mbH");
  add(0x10fc, "I-O Data Device, Inc.");
  add(0x10fd, "Soyo Computer, Inc");
  add(0x10fe, "Fast Multimedia AG");
  add(0x10ff, "NCube");
  add(0x1100, "Jazz Multimedia");
  add(0x1101, "Initio Corporation");
  add(0x1102, "Creative Labs");
  add(0x1103, "HighPoint Technologies, Inc.");
  add(0x1104, "RasterOps Corp.");
  add(0x1105, "Sigma Designs, Inc.");
  add(0x1106, "VIA Technologies, Inc.");
  add(0x1107, "Stratus Computers");
  add(0x1108, "Proteon, Inc.");
  add(0x1109, "Cogent Data Technologies, Inc.");
  add(0x110a, "Siemens AG");
  add(0x110b, "Chromatic Research Inc.");
  add(0x110c, "Mini-Max Technology, Inc.");
  add(0x110d, "Znyx Advanced Systems");
  add(0x110e, "CPU Technology");
  add(0x110f, "Ross Technology");
  add(0x1110, "Powerhouse Systems");
  add(0x1111, "Santa Cruz Operation");
  add(0x1112, "Osicom Technologies Inc");
  add(0x1113, "Accton Technology Corporation");
  add(0x1114, "Atmel Corporation");
  add(0x1115, "3D Labs");
  add(0x1116, "Data Translation");
  add(0x1117, "Datacube, Inc");
  add(0x1118, "Berg Electronics");
  add(0x1119, "ICP Vortex Computersysteme GmbH");
  add(0x111a, "Efficient Networks, Inc");
  add(0x111b, "Teledyne Electronic Systems");
  add(0x111c, "Tricord Systems Inc.");
  add(0x111d, "Integrated Device Technology, Inc. [IDT]");
  add(0x111e, "Eldec");
  add(0x111f, "Precision Digital Images");
  add(0x1120, "EMC Corporation");
  add(0x1121, "Zilog");
  add(0x1122, "Multi-tech Systems, Inc.");
  add(0x1123, "Excellent Design, Inc.");
  add(0x1124, "Leutron Vision AG");
  add(0x1125, "Eurocore");
  add(0x1126, "Vigra");
  add(0x1127, "FORE Systems Inc");
  add(0x1129, "Firmworks");
  add(0x112a, "Hermes Electronics Company, Ltd.");
  add(0x112b, "Linotype - Hell AG");
  add(0x112c, "Zenith Data Systems");
  add(0x112d, "Ravicad");
  add(0x112e, "Infomedia Microelectronics Inc.");
  add(0x112f, "Dalsa Inc.");
  add(0x1130, "Computervision");
  add(0x1131, "Philips Semiconductors");
  add(0x1132, "Mitel Corp.");
  add(0x1133, "Dialogic Corporation");
  add(0x1134, "Mercury Computer Systems");
  add(0x1135, "Fuji Xerox Co Ltd");
  add(0x1136, "Momentum Data Systems");
  add(0x1137, "Cisco Systems Inc");
  add(0x1138, "Ziatech Corporation");
  add(0x1139, "Dynamic Pictures, Inc");
  add(0x113a, "FWB Inc");
  add(0x113b, "Network Computing Devices");
  add(0x113c, "Cyclone Microsystems, Inc.");
  add(0x113d, "Leading Edge Products Inc");
  add(0x113e, "Sanyo Electric Co - Computer Engineering Dept");
  add(0x113f, "Equinox Systems, Inc.");
  add(0x1140, "Intervoice Inc");
  add(0x1141, "Crest Microsystem Inc");
  add(0x1142, "Alliance Semiconductor Corporation");
  add(0x1143, "NetPower, Inc");
  add(0x1144, "Cincinnati Milacron");
  add(0x1145, "Workbit Corporation");
  add(0x1146, "Force Computers");
  add(0x1147, "Interface Corp");
  add(0x1148, "SysKonnect");
  add(0x1149, "Win System Corporation");
  add(0x114a, "VMIC");
  add(0x114b, "Canopus Co., Ltd");
  add(0x114c, "Annabooks");
  add(0x114d, "IC Corporation");
  add(0x114e, "Nikon Systems Inc");
  add(0x114f, "Digi International");
  add(0x1150, "Thinking Machines Corp");
  add(0x1151, "JAE Electronics Inc.");
  add(0x1152, "Megatek");
  add(0x1153, "Land Win Electronic Corp");
  add(0x1154, "Melco Inc");
  add(0x1155, "Pine Technology Ltd");
  add(0x1156, "Periscope Engineering");
  add(0x1157, "Avsys Corporation");
  add(0x1158, "Voarx R & D Inc");
  add(0x1159, "Mutech Corp");
  add(0x115a, "Harlequin Ltd");
  add(0x115b, "Parallax Graphics");
  add(0x115c, "Photron Ltd.");
  add(0x115d, "Xircom");
  add(0x115e, "Peer Protocols Inc");
  add(0x115f, "Maxtor Corporation");
  add(0x1160, "Megasoft Inc");
  add(0x1161, "PFU Limited");
  add(0x1162, "OA Laboratory Co Ltd");
  add(0x1163, "Rendition");
  add(0x1164, "Advanced Peripherals Technologies");
  add(0x1165, "Imagraph Corporation");
  add(0x1166, "Broadcom");
  add(0x1167, "Mutoh Industries Inc");
  add(0x1168, "Thine Electronics Inc");
  add(0x1169, "Centre for Development of Advanced Computing");
  add(0x116a, "Luminex Software, Inc.");
  add(0x116b, "Connectware Inc");
  add(0x116c, "Intelligent Resources Integrated Systems");
  add(0x116d, "Martin-Marietta");
  add(0x116e, "Electronics for Imaging");
  add(0x116f, "Workstation Technology");
  add(0x1170, "Inventec Corporation");
  add(0x1171, "Loughborough Sound Images Plc");
  add(0x1172, "Altera Corporation");
  add(0x1173, "Adobe Systems, Inc");
  add(0x1174, "Bridgeport Machines");
  add(0x1175, "Mitron Computer Inc.");
  add(0x1176, "SBE Incorporated");
  add(0x1177, "Silicon Engineering");
  add(0x1178, "Alfa, Inc.");
  add(0x1179, "Toshiba America Info Systems");
  add(0x117a, "A-Trend Technology");
  add(0x117b, "L G Electronics, Inc.");
  add(0x117c, "ATTO Technology, Inc.");
  add(0x117d, "Becton & Dickinson");
  add(0x117e, "T/R Systems");
  add(0x117f, "Integrated Circuit Systems");
  add(0x1180, "Ricoh Co Ltd");
  add(0x1181, "Telmatics International");
  add(0x1183, "Fujikura Ltd");
  add(0x1184, "Forks Inc");
  add(0x1185, "Dataworld International Ltd");
  add(0x1186, "D-Link System Inc");
  add(0x1187, "Advanced Technology Laboratories, Inc.");
  add(0x1188, "Shima Seiki Manufacturing Ltd.");
  add(0x1189, "Matsushita Electronics Co Ltd");
  add(0x118a, "Hilevel Technology");
  add(0x118b, "Hypertec Pty Limited");
  add(0x118c, "Corollary, Inc");
  add(0x118d, "BitFlow Inc");
  add(0x118e, "Hermstedt GmbH");
  add(0x118f, "Green Logic");
  add(0x1190, "Tripace");
  add(0x1191, "Artop Electronic Corp");
  add(0x1192, "Densan Company Ltd");
  add(0x1193, "Zeitnet Inc.");
  add(0x1194, "Toucan Technology");
  add(0x1195, "Ratoc System Inc");
  add(0x1196, "Hytec Electronics Ltd");
  add(0x1197, "Gage Applied Sciences, Inc.");
  add(0x1198, "Lambda Systems Inc");
  add(0x1199, "Attachmate Corporation");
  add(0x119a, "Mind Share, Inc.");
  add(0x119b, "Omega Micro Inc.");
  add(0x119c, "Information Technology Inst.");
  add(0x119d, "Bug, Inc. Sapporo Japan");
  add(0x119e, "Fujitsu Microelectronics Ltd.");
  add(0x119f, "Bull HN Information Systems");
  add(0x11a0, "Convex Computer Corporation");
  add(0x11a1, "Hamamatsu Photonics K.K.");
  add(0x11a2, "Sierra Research and Technology");
  add(0x11a3, "Deuretzbacher GmbH & Co. Eng. KG");
  add(0x11a4, "Barco Graphics NV");
  add(0x11a5, "Microunity Systems Eng. Inc");
  add(0x11a6, "Pure Data Ltd.");
  add(0x11a7, "Power Computing Corp.");
  add(0x11a8, "Systech Corp.");
  add(0x11a9, "InnoSys Inc.");
  add(0x11aa, "Actel");
  add(0x11ab, "Marvell Technology Group Ltd.");
  add(0x11ac, "Canon Information Systems Research Aust.");
  add(0x11ad, "Lite-On Communications Inc");
  add(0x11ae, "Aztech System Ltd");
  add(0x11af, "Avid Technology Inc.");
  add(0x11b0, "V3 Semiconductor Inc.");
  add(0x11b1, "Apricot Computers");
  add(0x11b2, "Eastman Kodak");
  add(0x11b3, "Barr Systems Inc.");
  add(0x11b4, "Leitch Technology International");
  add(0x11b5, "Radstone Technology Plc");
  add(0x11b6, "United Video Corp");
  add(0x11b7, "Motorola");
  add(0x11b8, "XPoint Technologies, Inc");
  add(0x11b9, "Pathlight Technology Inc.");
  add(0x11ba, "Videotron Corp");
  add(0x11bb, "Pyramid Technology");
  add(0x11bc, "Network Peripherals Inc");
  add(0x11bd, "Pinnacle Systems Inc.");
  add(0x11be, "International Microcircuits Inc");
  add(0x11bf, "Astrodesign, Inc.");
  add(0x11c0, "Hewlett Packard");
  add(0x11c1, "LSI Corporation");
  add(0x11c2, "Sand Microelectronics");
  add(0x11c3, "NEC Corporation");
  add(0x11c4, "Document Technologies, Inc");
  add(0x11c5, "Shiva Corporation");
  add(0x11c6, "Dainippon Screen Mfg. Co. Ltd");
  add(0x11c7, "D.C.M. Data Systems");
  add(0x11c8, "Dolphin Interconnect Solutions AS");
  add(0x11c9, "Magma");
  add(0x11ca, "LSI Systems, Inc");
  add(0x11cb, "Specialix Research Ltd.");
  add(0x11cc, "Michels & Kleberhoff Computer GmbH");
  add(0x11cd, "HAL Computer Systems, Inc.");
  add(0x11ce, "Netaccess");
  add(0x11cf, "Pioneer Electronic Corporation");
  add(0x11d0, "Lockheed Martin Federal Systems-Manassas");
  add(0x11d1, "Auravision");
  add(0x11d2, "Intercom Inc.");
  add(0x11d3, "Trancell Systems Inc");
  add(0x11d4, "Analog Devices");
  add(0x11d5, "Ikon Corporation");
  add(0x11d6, "Tekelec Telecom");
  add(0x11d7, "Trenton Technology, Inc.");
  add(0x11d8, "Image Technologies Development");
  add(0x11d9, "TEC Corporation");
  add(0x11da, "Novell");
  add(0x11db, "Sega Enterprises Ltd");
  add(0x11dc, "Questra Corporation");
  add(0x11dd, "Crosfield Electronics Limited");
  add(0x11de, "Zoran Corporation");
  add(0x11df, "New Wave PDG");
  add(0x11e0, "Cray Communications A/S");
  add(0x11e1, "GEC Plessey Semi Inc.");
  add(0x11e2, "Samsung Information Systems America");
  add(0x11e3, "Quicklogic Corporation");
  add(0x11e4, "Second Wave Inc");
  add(0x11e5, "IIX Consulting");
  add(0x11e6, "Mitsui-Zosen System Research");
  add(0x11e7, "Toshiba America, Elec. Company");
  add(0x11e8, "Digital Processing Systems Inc.");
  add(0x11e9, "Highwater Designs Ltd.");
  add(0x11ea, "Elsag Bailey");
  add(0x11eb, "Formation Inc.");
  add(0x11ec, "Coreco Inc");
  add(0x11ed, "Mediamatics");
  add(0x11ee, "Dome Imaging Systems Inc");
  add(0x11ef, "Nicolet Technologies B.V.");
  add(0x11f0, "Compu-Shack");
  add(0x11f1, "Symbios Logic Inc");
  add(0x11f2, "Picture Tel Japan K.K.");
  add(0x11f3, "Keithley Metrabyte");
  add(0x11f4, "Kinetic Systems Corporation");
  add(0x11f5, "Computing Devices International");
  add(0x11f6, "Compex");
  add(0x11f7, "Scientific Atlanta");
  add(0x11f8, "PMC-Sierra Inc.");
  add(0x11f9, "I-Cube Inc");
  add(0x11fa, "Kasan Electronics Company, Ltd.");
  add(0x11fb, "Datel Inc");
  add(0x11fc, "Silicon Magic");
  add(0x11fd, "High Street Consultants");
  add(0x11fe, "Comtrol Corporation");
  add(0x11ff, "Scion Corporation");
  add(0x1200, "CSS Corporation");
  add(0x1201, "Vista Controls Corp");
  add(0x1202, "Network General Corp.");
  add(0x1203, "Bayer Corporation, Agfa Division");
  add(0x1204, "Lattice Semiconductor Corporation");
  add(0x1205, "Array Corporation");
  add(0x1206, "Amdahl Corporation");
  add(0x1208, "Parsytec GmbH");
  add(0x1209, "SCI Systems Inc");
  add(0x120a, "Synaptel");
  add(0x120b, "Adaptive Solutions");
  add(0x120c, "Technical Corp.");
  add(0x120d, "Compression Labs, Inc.");
  add(0x120e, "Cyclades Corporation");
  add(0x120f, "Essential Communications");
  add(0x1210, "Hyperparallel Technologies");
  add(0x1211, "Braintech Inc");
  add(0x1212, "Kingston Technology Corp.");
  add(0x1213, "Applied Intelligent Systems, Inc.");
  add(0x1214, "Performance Technologies, Inc.");
  add(0x1215, "Interware Co., Ltd");
  add(0x1216, "Purup Prepress A/S");
  add(0x1217, "O2 Micro, Inc.");
  add(0x1218, "Hybricon Corp.");
  add(0x1219, "First Virtual Corporation");
  add(0x121a, "3Dfx Interactive, Inc.");
  add(0x121b, "Advanced Telecommunications Modules");
  add(0x121c, "Nippon Texaco., Ltd");
  add(0x121d, "LiPPERT ADLINK Technology GmbH");
  add(0x121e, "CSPI");
  add(0x121f, "Arcus Technology, Inc.");
  add(0x1220, "Ariel Corporation");
  add(0x1221, "Contec Co., Ltd");
  add(0x1222, "Ancor Communications, Inc.");
  add(0x1223, "Artesyn Communication Products");
  add(0x1224, "Interactive Images");
  add(0x1225, "Power I/O, Inc.");
  add(0x1227, "Tech-Source");
  add(0x1228, "Norsk Elektro Optikk A/S");
  add(0x1229, "Data Kinesis Inc.");
  add(0x122a, "Integrated Telecom");
  add(0x122b, "LG Industrial Systems Co., Ltd");
  add(0x122c, "Sican GmbH");
  add(0x122d, "Aztech System Ltd");
  add(0x122e, "Xyratex");
  add(0x122f, "Andrew Corporation");
  add(0x1230, "Fishcamp Engineering");
  add(0x1231, "Woodward McCoach, Inc.");
  add(0x1232, "GPT Limited");
  add(0x1233, "Bus-Tech, Inc.");
  add(0x1235, "Risq Modular Systems, Inc.");
  add(0x1236, "Sigma Designs Corporation");
  add(0x1237, "Alta Technology Corporation");
  add(0x1238, "Adtran");
  add(0x1239, "3DO Company");
  add(0x123a, "Visicom Laboratories, Inc.");
  add(0x123b, "Seeq Technology, Inc.");
  add(0x123c, "Century Systems, Inc.");
  add(0x123d, "Engineering Design Team, Inc.");
  add(0x123e, "Simutech, Inc.");
  add(0x123f, "LSI Logic");
  add(0x1240, "Marathon Technologies Corp.");
  add(0x1241, "DSC Communications");
  add(0x1242, "JNI Corporation");
  add(0x1243, "Delphax");
  add(0x1244, "AVM GmbH");
  add(0x1245, "A.P.D., S.A.");
  add(0x1246, "Dipix Technologies, Inc.");
  add(0x1247, "Xylon Research, Inc.");
  add(0x1248, "Central Data Corporation");
  add(0x1249, "Samsung Electronics Co., Ltd.");
  add(0x124a, "AEG Electrocom GmbH");
  add(0x124b, "SBS/Greenspring Modular I/O");
  add(0x124c, "Solitron Technologies, Inc.");
  add(0x124d, "Stallion Technologies, Inc.");
  add(0x124e, "Cylink");
  add(0x124f, "Infortrend Technology, Inc.");
  add(0x1250, "Hitachi Microcomputer System Ltd");
  add(0x1251, "VLSI Solutions Oy");
  add(0x1253, "Guzik Technical Enterprises");
  add(0x1254, "Linear Systems Ltd.");
  add(0x1255, "Optibase Ltd");
  add(0x1256, "Perceptive Solutions, Inc.");
  add(0x1257, "Vertex Networks, Inc.");
  add(0x1258, "Gilbarco, Inc.");
  add(0x1259, "Allied Telesis");
  add(0x125a, "ABB Power Systems");
  add(0x125b, "Asix Electronics Corporation");
  add(0x125c, "Aurora Technologies, Inc.");
  add(0x125d, "ESS Technology");
  add(0x125e, "Specialvideo Engineering SRL");
  add(0x125f, "Concurrent Technologies, Inc.");
  add(0x1260, "Intersil Corporation");
  add(0x1261, "Matsushita-Kotobuki Electronics Industries, Ltd.");
  add(0x1262, "ES Computer Company, Ltd.");
  add(0x1263, "Sonic Solutions");
  add(0x1264, "Aval Nagasaki Corporation");
  add(0x1265, "Casio Computer Co., Ltd.");
  add(0x1266, "Microdyne Corporation");
  add(0x1267, "S. A. Telecommunications");
  add(0x1268, "Tektronix");
  add(0x1269, "Thomson-CSF/TTM");
  add(0x126a, "Lexmark International, Inc.");
  add(0x126b, "Adax, Inc.");
  add(0x126c, "Northern Telecom");
  add(0x126d, "Splash Technology, Inc.");
  add(0x126e, "Sumitomo Metal Industries, Ltd.");
  add(0x126f, "Silicon Motion, Inc.");
  add(0x1270, "Olympus Optical Co., Ltd.");
  add(0x1271, "GW Instruments");
  add(0x1272, "Telematics International");
  add(0x1273, "Hughes Network Systems");
  add(0x1274, "Ensoniq");
  add(0x1275, "Network Appliance Corporation");
  add(0x1276, "Switched Network Technologies, Inc.");
  add(0x1277, "Comstream");
  add(0x1278, "Transtech Parallel Systems Ltd.");
  add(0x1279, "Transmeta Corporation");
  add(0x127a, "Rockwell International");
  add(0x127b, "Pixera Corporation");
  add(0x127c, "Crosspoint Solutions, Inc.");
  add(0x127d, "Vela Research");
  add(0x127e, "Winnov, L.P.");
  add(0x127f, "Fujifilm");
  add(0x1280, "Photoscript Group Ltd.");
  add(0x1281, "Yokogawa Electric Corporation");
  add(0x1282, "Davicom Semiconductor, Inc.");
  add(0x1283, "Integrated Technology Express, Inc.");
  add(0x1284, "Sahara Networks, Inc.");
  add(0x1285, "Platform Technologies, Inc.");
  add(0x1286, "Mazet GmbH");
  add(0x1287, "M-Pact, Inc.");
  add(0x1288, "Timestep Corporation");
  add(0x1289, "AVC Technology, Inc.");
  add(0x128a, "Asante Technologies, Inc.");
  add(0x128b, "Transwitch Corporation");
  add(0x128c, "Retix Corporation");
  add(0x128d, "G2 Networks, Inc.");
  add(0x128e, "Hoontech Corporation/Samho Multi Tech Ltd.");
  add(0x128f, "Tateno Dennou, Inc.");
  add(0x1290, "Sord Computer Corporation");
  add(0x1291, "NCS Computer Italia");
  add(0x1292, "Tritech Microelectronics Inc");
  add(0x1293, "Media Reality Technology");
  add(0x1294, "Rhetorex, Inc.");
  add(0x1295, "Imagenation Corporation");
  add(0x1296, "Kofax Image Products");
  add(0x1297, "Holco Enterprise Co, Ltd/Shuttle Computer");
  add(0x1298, "Spellcaster Telecommunications Inc.");
  add(0x1299, "Knowledge Technology Lab.");
  add(0x129a, "VMetro, inc.");
  add(0x129b, "Image Access");
  add(0x129c, "Jaycor");
  add(0x129d, "Compcore Multimedia, Inc.");
  add(0x129e, "Victor Company of Japan, Ltd.");
  add(0x129f, "OEC Medical Systems, Inc.");
  add(0x12a0, "Allen-Bradley Company");
  add(0x12a1, "Simpact Associates, Inc.");
  add(0x12a2, "Newgen Systems Corporation");
  add(0x12a3, "Lucent Technologies");
  add(0x12a4, "NTT Electronics Technology Company");
  add(0x12a5, "Vision Dynamics Ltd.");
  add(0x12a6, "Scalable Networks, Inc.");
  add(0x12a7, "AMO GmbH");
  add(0x12a8, "News Datacom");
  add(0x12a9, "Xiotech Corporation");
  add(0x12aa, "SDL Communications, Inc.");
  add(0x12ab, "Yuan Yuan Enterprise Co., Ltd.");
  add(0x12ac, "Measurex Corporation");
  add(0x12ad, "Multidata GmbH");
  add(0x12ae, "Alteon Networks Inc.");
  add(0x12af, "TDK USA Corp");
  add(0x12b0, "Jorge Scientific Corp");
  add(0x12b1, "GammaLink");
  add(0x12b2, "General Signal Networks");
  add(0x12b3, "Inter-Face Co Ltd");
  add(0x12b4, "FutureTel Inc");
  add(0x12b5, "Granite Systems Inc.");
  add(0x12b6, "Natural Microsystems");
  add(0x12b7, "Cognex Modular Vision Systems Div. - Acumen Inc.");
  add(0x12b8, "Korg");
  add(0x12b9, "3Com Corp, Modem Division");
  add(0x12ba, "BittWare, Inc.");
  add(0x12bb, "Nippon Unisoft Corporation");
  add(0x12bc, "Array Microsystems");
  add(0x12bd, "Computerm Corp.");
  add(0x12be, "Anchor Chips Inc.");
  add(0x12bf, "Fujifilm Microdevices");
  add(0x12c0, "Infimed");
  add(0x12c1, "GMM Research Corp");
  add(0x12c2, "Mentec Limited");
  add(0x12c3, "Holtek Microelectronics Inc");
  add(0x12c4, "Connect Tech Inc");
  add(0x12c5, "Picture Elements Incorporated");
  add(0x12c6, "Mitani Corporation");
  add(0x12c7, "Dialogic Corp");
  add(0x12c8, "G Force Co, Ltd");
  add(0x12c9, "Gigi Operations");
  add(0x12ca, "Integrated Computing Engines");
  add(0x12cb, "Antex Electronics Corporation");
  add(0x12cc, "Pluto Technologies International");
  add(0x12cd, "Aims Lab");
  add(0x12ce, "Netspeed Inc.");
  add(0x12cf, "Prophet Systems, Inc.");
  add(0x12d0, "GDE Systems, Inc.");
  add(0x12d1, "PSITech");
  add(0x12d2, "NVidia / SGS Thomson (Joint Venture)");
  add(0x12d3, "Vingmed Sound A/S");
  add(0x12d4, "Ulticom (Formerly DGM&S)");
  add(0x12d5, "Equator Technologies Inc");
  add(0x12d6, "Analogic Corp");
  add(0x12d7, "Biotronic SRL");
  add(0x12d8, "Pericom Semiconductor");
  add(0x12d9, "Aculab PLC");
  add(0x12da, "True Time Inc.");
  add(0x12db, "Annapolis Micro Systems, Inc");
  add(0x12dc, "Symicron Computer Communication Ltd.");
  add(0x12dd, "Management Graphics");
  add(0x12de, "Rainbow Technologies");
  add(0x12df, "SBS Technologies Inc");
  add(0x12e0, "Chase Research");
  add(0x12e1, "Nintendo Co, Ltd");
  add(0x12e2, "Datum Inc. Bancomm-Timing Division");
  add(0x12e3, "Imation Corp - Medical Imaging Systems");
  add(0x12e4, "Brooktrout Technology Inc");
  add(0x12e5, "Apex Semiconductor Inc");
  add(0x12e6, "Cirel Systems");
  add(0x12e7, "Sunsgroup Corporation");
  add(0x12e8, "Crisc Corp");
  add(0x12e9, "GE Spacenet");
  add(0x12ea, "Zuken");
  add(0x12eb, "Aureal Semiconductor");
  add(0x12ec, "3A International, Inc.");
  add(0x12ed, "Optivision Inc.");
  add(0x12ee, "Orange Micro");
  add(0x12ef, "Vienna Systems");
  add(0x12f0, "Pentek");
  add(0x12f1, "Sorenson Vision Inc");
  add(0x12f2, "Gammagraphx, Inc.");
  add(0x12f3, "Radstone Technology");
  add(0x12f4, "Megatel");
  add(0x12f5, "Forks");
  add(0x12f6, "Dawson France");
  add(0x12f7, "Cognex");
  add(0x12f8, "Electronic Design GmbH");
  add(0x12f9, "Four Fold Ltd");
  add(0x12fb, "Spectrum Signal Processing");
  add(0x12fc, "Capital Equipment Corp");
  add(0x12fd, "I2S");
  add(0x12fe, "ESD Electronic System Design GmbH");
  add(0x12ff, "Lexicon");
  add(0x1300, "Harman International Industries Inc");
  add(0x1302, "Computer Sciences Corp");
  add(0x1303, "Innovative Integration");
  add(0x1304, "Juniper Networks");
  add(0x1305, "Netphone, Inc");
  add(0x1306, "Duet Technologies");
  add(0x1307, "Measurement Computing");
  add(0x1308, "Jato Technologies Inc.");
  add(0x1309, "AB Semiconductor Ltd");
  add(0x130a, "Mitsubishi Electric Microcomputer");
  add(0x130b, "Colorgraphic Communications Corp");
  add(0x130c, "Ambex Technologies, Inc");
  add(0x130d, "Accelerix Inc");
  add(0x130e, "Yamatake-Honeywell Co. Ltd");
  add(0x130f, "Advanet Inc");
  add(0x1310, "Gespac");
  add(0x1311, "Videoserver, Inc");
  add(0x1312, "Acuity Imaging, Inc");
  add(0x1313, "Yaskawa Electric Co.");
  add(0x1315, "Wavesat");
  add(0x1316, "Teradyne Inc");
  add(0x1317, "ADMtek");
  add(0x1318, "Packet Engines Inc.");
  add(0x1319, "Fortemedia, Inc");
  add(0x131a, "Finisar Corp.");
  add(0x131c, "Nippon Electro-Sensory Devices Corp");
  add(0x131d, "Sysmic, Inc.");
  add(0x131e, "Xinex Networks Inc");
  add(0x131f, "Siig Inc");
  add(0x1320, "Crypto AG");
  add(0x1321, "Arcobel Graphics BV");
  add(0x1322, "MTT Co., Ltd");
  add(0x1323, "Dome Inc");
  add(0x1324, "Sphere Communications");
  add(0x1325, "Salix Technologies, Inc");
  add(0x1326, "Seachange international");
  add(0x1327, "Voss scientific");
  add(0x1328, "quadrant international");
  add(0x1329, "Productivity Enhancement");
  add(0x132a, "Microcom Inc.");
  add(0x132b, "Broadband Technologies");
  add(0x132c, "Micrel Inc");
  add(0x132d, "Integrated Silicon Solution, Inc.");
  add(0x1330, "MMC Networks");
  add(0x1331, "RadiSys Corporation");
  add(0x1332, "Micro Memory");
  add(0x1334, "Redcreek Communications, Inc");
  add(0x1335, "Videomail, Inc");
  add(0x1337, "Third Planet Publishing");
  add(0x1338, "BT Electronics");
  add(0x133a, "Vtel Corp");
  add(0x133b, "Softcom Microsystems");
  add(0x133c, "Holontech Corp");
  add(0x133d, "SS Technologies");
  add(0x133e, "Virtual Computer Corp");
  add(0x133f, "SCM Microsystems");
  add(0x1340, "Atalla Corp");
  add(0x1341, "Kyoto Microcomputer Co");
  add(0x1342, "Promax Systems Inc");
  add(0x1343, "Phylon Communications Inc");
  add(0x1344, "Micron Technology Inc");
  add(0x1345, "Arescom Inc");
  add(0x1347, "Odetics");
  add(0x1349, "Sumitomo Electric Industries, Ltd.");
  add(0x134a, "DTC Technology Corp.");
  add(0x134b, "ARK Research Corp.");
  add(0x134c, "Chori Joho System Co. Ltd");
  add(0x134d, "PCTel Inc");
  add(0x134e, "CSTI");
  add(0x134f, "Algo System Co Ltd");
  add(0x1350, "Systec Co. Ltd");
  add(0x1351, "Sonix Inc");
  add(0x1353, "Vierling Communication SAS");
  add(0x1354, "Dwave System Inc");
  add(0x1355, "Kratos Analytical Ltd");
  add(0x1356, "The Logical Co");
  add(0x1359, "Prisa Networks");
  add(0x135a, "Brain Boxes");
  add(0x135b, "Giganet Inc");
  add(0x135c, "Quatech Inc");
  add(0x135d, "ABB Network Partner AB");
  add(0x135e, "Sealevel Systems Inc");
  add(0x135f, "I-Data International A-S");
  add(0x1360, "Meinberg Funkuhren");
  add(0x1361, "Soliton Systems K.K.");
  add(0x1362, "Fujifacom Corporation");
  add(0x1363, "Phoenix Technology Ltd");
  add(0x1364, "ATM Communications Inc");
  add(0x1365, "Hypercope GmbH");
  add(0x1366, "Teijin Seiki Co. Ltd");
  add(0x1367, "Hitachi Zosen Corporation");
  add(0x1368, "Skyware Corporation");
  add(0x1369, "Digigram");
  add(0x136a, "High Soft Tech");
  add(0x136b, "Kawasaki Steel Corporation");
  add(0x136c, "Adtek System Science Co Ltd");
  add(0x136d, "Gigalabs Inc");
  add(0x136f, "Applied Magic Inc");
  add(0x1370, "ATL Products");
  add(0x1371, "CNet Technology Inc");
  add(0x1373, "Silicon Vision Inc");
  add(0x1374, "Silicom Ltd.");
  add(0x1375, "Argosystems Inc");
  add(0x1376, "LMC");
  add(0x1377, "Electronic Equipment Production & Distribution GmbH");
  add(0x1378, "Telemann Co. Ltd");
  add(0x1379, "Asahi Kasei Microsystems Co Ltd");
  add(0x137a, "Mark of the Unicorn Inc");
  add(0x137b, "PPT Vision");
  add(0x137c, "Iwatsu Electric Co Ltd");
  add(0x137d, "Dynachip Corporation");
  add(0x137e, "Patriot Scientific Corporation");
  add(0x137f, "Japan Satellite Systems Inc");
  add(0x1380, "Sanritz Automation Co Ltd");
  add(0x1381, "Brains Co. Ltd");
  add(0x1382, "Marian - Electronic & Software");
  add(0x1383, "Controlnet Inc");
  add(0x1384, "Reality Simulation Systems Inc");
  add(0x1385, "Netgear");
  add(0x1386, "Video Domain Technologies");
  add(0x1387, "Systran Corp");
  add(0x1388, "Hitachi Information Technology Co Ltd");
  add(0x1389, "Applicom International");
  add(0x138a, "Fusion Micromedia Corp");
  add(0x138b, "Tokimec Inc");
  add(0x138c, "Silicon Reality");
  add(0x138d, "Future Techno Designs pte Ltd");
  add(0x138e, "Basler GmbH");
  add(0x138f, "Patapsco Designs Inc");
  add(0x1390, "Concept Development Inc");
  add(0x1391, "Development Concepts Inc");
  add(0x1392, "Medialight Inc");
  add(0x1393, "Moxa Technologies Co Ltd");
  add(0x1394, "Level One Communications");
  add(0x1395, "Ambicom Inc");
  add(0x1396, "Cipher Systems Inc");
  add(0x1397, "Cologne Chip Designs GmbH");
  add(0x1398, "Clarion co. Ltd");
  add(0x1399, "Rios systems Co Ltd");
  add(0x139a, "Alacritech Inc");
  add(0x139b, "Mediasonic Multimedia Systems Ltd");
  add(0x139c, "Quantum 3d Inc");
  add(0x139d, "EPL limited");
  add(0x139e, "Media4");
  add(0x139f, "Aethra s.r.l.");
  add(0x13a0, "Crystal Group Inc");
  add(0x13a1, "Kawasaki Heavy Industries Ltd");
  add(0x13a2, "Ositech Communications Inc");
  add(0x13a3, "Hifn Inc.");
  add(0x13a4, "Rascom Inc");
  add(0x13a5, "Audio Digital Imaging Inc");
  add(0x13a6, "Videonics Inc");
  add(0x13a7, "Teles AG");
  add(0x13a8, "Exar Corp.");
  add(0x13a9, "Siemens Medical Systems, Ultrasound Group");
  add(0x13aa, "Broadband Networks Inc");
  add(0x13ab, "Arcom Control Systems Ltd");
  add(0x13ac, "Motion Media Technology Ltd");
  add(0x13ad, "Nexus Inc");
  add(0x13ae, "ALD Technology Ltd");
  add(0x13af, "T.Sqware");
  add(0x13b0, "Maxspeed Corp");
  add(0x13b1, "Tamura corporation");
  add(0x13b2, "Techno Chips Co. Ltd");
  add(0x13b3, "Lanart Corporation");
  add(0x13b4, "Wellbean Co Inc");
  add(0x13b5, "ARM");
  add(0x13b6, "Dlog GmbH");
  add(0x13b7, "Logic Devices Inc");
  add(0x13b8, "Nokia Telecommunications oy");
  add(0x13b9, "Elecom Co Ltd");
  add(0x13ba, "Oxford Instruments");
  add(0x13bb, "Sanyo Technosound Co Ltd");
  add(0x13bc, "Bitran Corporation");
  add(0x13bd, "Sharp corporation");
  add(0x13be, "Miroku Jyoho Service Co. Ltd");
  add(0x13bf, "Sharewave Inc");
  add(0x13c0, "Microgate Corporation");
  add(0x13c1, "3ware Inc");
  add(0x13c2, "Technotrend Systemtechnik GmbH");
  add(0x13c3, "Janz Computer AG");
  add(0x13c4, "Phase Metrics");
  add(0x13c5, "Alphi Technology Corp");
  add(0x13c6, "Condor Engineering Inc");
  add(0x13c7, "Blue Chip Technology Ltd");
  add(0x13c8, "Apptech Inc");
  add(0x13c9, "Eaton Corporation");
  add(0x13ca, "Iomega Corporation");
  add(0x13cb, "Yano Electric Co Ltd");
  add(0x13cc, "Metheus Corporation");
  add(0x13cd, "Compatible Systems Corporation");
  add(0x13ce, "Cocom A/S");
  add(0x13cf, "Studio Audio & Video Ltd");
  add(0x13d0, "Techsan Electronics Co Ltd");
  add(0x13d1, "Abocom Systems Inc");
  add(0x13d2, "Shark Multimedia Inc");
  add(0x13d4, "Graphics Microsystems Inc");
  add(0x13d5, "Media 100 Inc");
  add(0x13d6, "K.I. Technology Co Ltd");
  add(0x13d7, "Toshiba Engineering Corporation");
  add(0x13d8, "Phobos corporation");
  add(0x13d9, "Apex PC Solutions Inc");
  add(0x13da, "Intresource Systems pte Ltd");
  add(0x13db, "Janich & Klass Computertechnik GmbH");
  add(0x13dc, "Netboost Corporation");
  add(0x13dd, "Multimedia Bundle Inc");
  add(0x13de, "ABB Robotics Products AB");
  add(0x13df, "E-Tech Inc");
  add(0x13e0, "GVC Corporation");
  add(0x13e1, "Silicom Multimedia Systems Inc");
  add(0x13e2, "Dynamics Research Corporation");
  add(0x13e3, "Nest Inc");
  add(0x13e4, "Calculex Inc");
  add(0x13e5, "Telesoft Design Ltd");
  add(0x13e6, "Argosy research Inc");
  add(0x13e7, "NAC Incorporated");
  add(0x13e8, "Chip Express Corporation");
  add(0x13e9, "Intraserver Technology Inc");
  add(0x13ea, "Dallas Semiconductor");
  add(0x13eb, "Hauppauge Computer Works Inc");
  add(0x13ec, "Zydacron Inc");
  add(0x13ed, "Raytheion E-Systems");
  add(0x13ee, "Hayes Microcomputer Products Inc");
  add(0x13ef, "Coppercom Inc");
  add(0x13f0, "Sundance Technology Inc / IC Plus Corp");
  add(0x13f1, "Oce' - Technologies B.V.");
  add(0x13f2, "Ford Microelectronics Inc");
  add(0x13f3, "Mcdata Corporation");
  add(0x13f4, "Troika Networks, Inc.");
  add(0x13f5, "Kansai Electric Co. Ltd");
  add(0x13f6, "C-Media Electronics Inc");
  add(0x13f7, "Wildfire Communications");
  add(0x13f8, "Ad Lib Multimedia Inc");
  add(0x13f9, "NTT Advanced Technology Corp.");
  add(0x13fa, "Pentland Systems Ltd");
  add(0x13fb, "Aydin Corp");
  add(0x13fc, "Computer Peripherals International");
  add(0x13fd, "Micro Science Inc");
  add(0x13fe, "Advantech Co. Ltd");
  add(0x13ff, "Silicon Spice Inc");
  add(0x1400, "Artx Inc");
  add(0x1401, "CR-Systems A/S");
  add(0x1402, "Meilhaus Electronic GmbH");
  add(0x1403, "Ascor Inc");
  add(0x1404, "Fundamental Software Inc");
  add(0x1405, "Excalibur Systems Inc");
  add(0x1406, "Oce' Printing Systems GmbH");
  add(0x1407, "Lava Computer mfg Inc");
  add(0x1408, "Aloka Co. Ltd");
  add(0x1409, "Timedia Technology Co Ltd");
  add(0x140a, "DSP Research Inc");
  add(0x140b, "GE Intelligent Platforms");
  add(0x140c, "Elmic Systems Inc");
  add(0x140d, "Matsushita Electric Works Ltd");
  add(0x140e, "Goepel Electronic GmbH");
  add(0x140f, "Salient Systems Corp");
  add(0x1410, "Midas lab Inc");
  add(0x1411, "Ikos Systems Inc");
  add(0x1412, "VIA Technologies Inc.");
  add(0x1413, "Addonics");
  add(0x1414, "Microsoft Corporation");
  add(0x1415, "Oxford Semiconductor Ltd");
  add(0x1416, "Multiwave Innovation pte Ltd");
  add(0x1417, "Convergenet Technologies Inc");
  add(0x1418, "Kyushu electronics systems Inc");
  add(0x1419, "Excel Switching Corp");
  add(0x141a, "Apache Micro Peripherals Inc");
  add(0x141b, "Zoom Telephonics Inc");
  add(0x141d, "Digitan Systems Inc");
  add(0x141e, "Fanuc Ltd");
  add(0x141f, "Visiontech Ltd");
  add(0x1420, "Psion Dacom plc");
  add(0x1421, "Ads Technologies Inc");
  add(0x1422, "Ygrec Systems Co Ltd");
  add(0x1423, "Custom Technology Corp.");
  add(0x1424, "Videoserver Connections");
  add(0x1425, "Chelsio Communications Inc");
  add(0x1426, "Storage Technology Corp.");
  add(0x1427, "Better On-Line Solutions");
  add(0x1428, "Edec Co Ltd");
  add(0x1429, "Unex Technology Corp.");
  add(0x142a, "Kingmax Technology Inc");
  add(0x142b, "Radiolan");
  add(0x142c, "Minton Optic Industry Co Ltd");
  add(0x142d, "Pix stream Inc");
  add(0x142e, "Vitec Multimedia");
  add(0x142f, "Radicom Research Inc");
  add(0x1430, "ITT Aerospace/Communications Division");
  add(0x1431, "Gilat Satellite Networks");
  add(0x1432, "Edimax Computer Co.");
  add(0x1433, "Eltec Elektronik GmbH");
  add(0x1435, "RTD Embedded Technologies, Inc.");
  add(0x1436, "CIS Technology Inc");
  add(0x1437, "Nissin Inc Co");
  add(0x1438, "Atmel-dream");
  add(0x1439, "Outsource Engineering & Mfg. Inc");
  add(0x143a, "Stargate Solutions Inc");
  add(0x143b, "Canon Research Center, America");
  add(0x143c, "Amlogic Inc");
  add(0x143d, "Tamarack Microelectronics Inc");
  add(0x143e, "Jones Futurex Inc");
  add(0x143f, "Lightwell Co Ltd - Zax Division");
  add(0x1440, "ALGOL Corp.");
  add(0x1441, "AGIE Ltd");
  add(0x1442, "Phoenix Contact GmbH & Co.");
  add(0x1443, "Unibrain S.A.");
  add(0x1444, "TRW");
  add(0x1445, "Logical DO Ltd");
  add(0x1446, "Graphin Co Ltd");
  add(0x1447, "AIM GmBH");
  add(0x1448, "Alesis Studio Electronics");
  add(0x1449, "TUT Systems Inc");
  add(0x144a, "Adlink Technology");
  add(0x144b, "Verint Systems Inc.");
  add(0x144c, "Catalina Research Inc");
  add(0x144d, "Samsung Electronics Co Ltd");
  add(0x144e, "OLITEC");
  add(0x144f, "Askey Computer Corp.");
  add(0x1450, "Octave Communications Ind.");
  add(0x1451, "SP3D Chip Design GmBH");
  add(0x1453, "MYCOM Inc");
  add(0x1454, "Altiga Networks");
  add(0x1455, "Logic Plus Plus Inc");
  add(0x1456, "Advanced Hardware Architectures");
  add(0x1457, "Nuera Communications Inc");
  add(0x1458, "Gigabyte Technology Co., Ltd");
  add(0x1459, "DOOIN Electronics");
  add(0x145a, "Escalate Networks Inc");
  add(0x145b, "PRAIM SRL");
  add(0x145c, "Cryptek");
  add(0x145d, "Gallant Computer Inc");
  add(0x145e, "Aashima Technology B.V.");
  add(0x145f, "Baldor Electric Company");
  add(0x1460, "DYNARC INC");
  add(0x1461, "Avermedia Technologies Inc");
  add(0x1462, "Micro-Star International Co., Ltd.");
  add(0x1463, "Fast Corporation");
  add(0x1464, "Interactive Circuits & Systems Ltd");
  add(0x1465, "GN NETTEST Telecom DIV.");
  add(0x1466, "Designpro Inc.");
  add(0x1467, "DIGICOM SPA");
  add(0x1468, "AMBIT Microsystem Corp.");
  add(0x1469, "Cleveland Motion Controls");
  add(0x146a, "IFR");
  add(0x146b, "Parascan Technologies Ltd");
  add(0x146c, "Ruby Tech Corp.");
  add(0x146d, "Tachyon, INC.");
  add(0x146e, "Williams Electronics Games, Inc.");
  add(0x146f, "Multi Dimensional Consulting Inc");
  add(0x1470, "Bay Networks");
  add(0x1471, "Integrated Telecom Express Inc");
  add(0x1472, "DAIKIN Industries, Ltd");
  add(0x1473, "ZAPEX Technologies Inc");
  add(0x1474, "Doug Carson & Associates");
  add(0x1475, "PICAZO Communications");
  add(0x1476, "MORTARA Instrument Inc");
  add(0x1477, "Net Insight");
  add(0x1478, "DIATREND Corporation");
  add(0x1479, "TORAY Industries Inc");
  add(0x147a, "FORMOSA Industrial Computing");
  add(0x147b, "ABIT Computer Corp.");
  add(0x147c, "AWARE, Inc.");
  add(0x147d, "Interworks Computer Products");
  add(0x147e, "Matsushita Graphic Communication Systems, Inc.");
  add(0x147f, "NIHON UNISYS, Ltd.");
  add(0x1480, "SCII Telecom");
  add(0x1481, "BIOPAC Systems Inc");
  add(0x1482, "ISYTEC - Integrierte Systemtechnik GmBH");
  add(0x1483, "LABWAY Corporation");
  add(0x1484, "Logic Corporation");
  add(0x1485, "ERMA - Electronic GmBH");
  add(0x1486, "L3 Communications Telemetry & Instrumentation");
  add(0x1487, "MARQUETTE Medical Systems");
  add(0x1488, "KONTRON Electronik GmBH");
  add(0x1489, "KYE Systems Corporation");
  add(0x148a, "OPTO");
  add(0x148b, "INNOMEDIALOGIC Inc.");
  add(0x148c, "Tul Corporation / PowerColor");
  add(0x148d, "DIGICOM Systems, Inc.");
  add(0x148e, "OSI Plus Corporation");
  add(0x148f, "Plant Equipment, Inc.");
  add(0x1490, "Stone Microsystems PTY Ltd.");
  add(0x1491, "ZEAL Corporation");
  add(0x1492, "Time Logic Corporation");
  add(0x1493, "MAKER Communications");
  add(0x1494, "WINTOP Technology, Inc.");
  add(0x1495, "TOKAI Communications Industry Co. Ltd");
  add(0x1496, "JOYTECH Computer Co., Ltd.");
  add(0x1497, "SMA Regelsysteme GmBH");
  add(0x1498, "TEWS Technologies GmbH");
  add(0x1499, "EMTEC CO., Ltd");
  add(0x149a, "ANDOR Technology Ltd");
  add(0x149b, "SEIKO Instruments Inc");
  add(0x149c, "OVISLINK Corp.");
  add(0x149d, "NEWTEK Inc");
  add(0x149e, "Mapletree Networks Inc.");
  add(0x149f, "LECTRON Co Ltd");
  add(0x14a0, "SOFTING GmBH");
  add(0x14a1, "Systembase Co Ltd");
  add(0x14a2, "Millennium Engineering Inc");
  add(0x14a3, "Maverick Networks");
  add(0x14a4, "Broadcom Corporation (Wrong ID)");
  add(0x14a5, "XIONICS Document Technologies Inc");
  add(0x14a6, "INOVA Computers GmBH & Co KG");
  add(0x14a7, "MYTHOS Systems Inc");
  add(0x14a8, "FEATRON Technologies Corporation");
  add(0x14a9, "HIVERTEC Inc");
  add(0x14aa, "Advanced MOS Technology Inc");
  add(0x14ab, "Mentor Graphics Corp.");
  add(0x14ac, "Novaweb Technologies Inc");
  add(0x14ad, "Time Space Radio AB");
  add(0x14ae, "CTI, Inc");
  add(0x14af, "Guillemot Corporation");
  add(0x14b0, "BST Communication Technology Ltd");
  add(0x14b1, "Nextcom K.K.");
  add(0x14b2, "ENNOVATE Networks Inc");
  add(0x14b3, "XPEED Inc");
  add(0x14b4, "PHILIPS Business Electronics B.V.");
  add(0x14b5, "Creamware GmBH");
  add(0x14b6, "Quantum Data Corp.");
  add(0x14b7, "PROXIM Inc");
  add(0x14b8, "Techsoft Technology Co Ltd");
  add(0x14b9, "Cisco Aironet Wireless Communications");
  add(0x14ba, "INTERNIX Inc.");
  add(0x14bb, "SEMTECH Corporation");
  add(0x14bc, "Globespan Semiconductor Inc.");
  add(0x14bd, "CARDIO Control N.V.");
  add(0x14be, "L3 Communications");
  add(0x14bf, "SPIDER Communications Inc.");
  add(0x14c0, "COMPAL Electronics Inc");
  add(0x14c1, "MYRICOM Inc.");
  add(0x14c2, "DTK Computer");
  add(0x14c3, "MEDIATEK Corp.");
  add(0x14c4, "IWASAKI Information Systems Co Ltd");
  add(0x14c5, "Automation Products AB");
  add(0x14c6, "Data Race Inc");
  add(0x14c7, "Modular Technology Holdings Ltd");
  add(0x14c8, "Turbocomm Tech. Inc.");
  add(0x14c9, "ODIN Telesystems Inc");
  add(0x14ca, "PE Logic Corp.");
  add(0x14cb, "Billionton Systems Inc");
  add(0x14cc, "NAKAYO Telecommunications Inc");
  add(0x14cd, "Universal Scientific Ind.");
  add(0x14ce, "Whistle Communications");
  add(0x14cf, "TEK Microsystems Inc.");
  add(0x14d0, "Ericsson Axe R & D");
  add(0x14d1, "Computer Hi-Tech Co Ltd");
  add(0x14d2, "Titan Electronics Inc");
  add(0x14d3, "CIRTECH (UK) Ltd");
  add(0x14d4, "Panacom Technology Corp");
  add(0x14d5, "Nitsuko Corporation");
  add(0x14d6, "Accusys Inc");
  add(0x14d7, "Hirakawa Hewtech Corp");
  add(0x14d8, "HOPF Elektronik GmBH");
  add(0x14d9, "Alliance Semiconductor Corporation");
  add(0x14da, "National Aerospace Laboratories");
  add(0x14db, "AFAVLAB Technology Inc");
  add(0x14dc, "Amplicon Liveline Ltd");
  add(0x14dd, "Boulder Design Labs Inc");
  add(0x14de, "Applied Integration Corporation");
  add(0x14df, "ASIC Communications Corp");
  add(0x14e1, "INVERTEX");
  add(0x14e2, "INFOLIBRIA");
  add(0x14e3, "AMTELCO");
  add(0x14e4, "Broadcom Corporation");
  add(0x14e5, "Pixelfusion Ltd");
  add(0x14e6, "SHINING Technology Inc");
  add(0x14e7, "3CX");
  add(0x14e8, "RAYCER Inc");
  add(0x14e9, "GARNETS System CO Ltd");
  add(0x14ea, "Planex Communications, Inc");
  add(0x14eb, "SEIKO EPSON Corp");
  add(0x14ec, "Agilent Technologies");
  add(0x14ed, "DATAKINETICS Ltd");
  add(0x14ee, "MASPRO KENKOH Corp");
  add(0x14ef, "CARRY Computer ENG. CO Ltd");
  add(0x14f0, "CANON RESEACH CENTRE FRANCE");
  add(0x14f1, "Conexant Systems, Inc.");
  add(0x14f2, "MOBILITY Electronics");
  add(0x14f3, "BroadLogic");
  add(0x14f4, "TOKYO Electronic Industry CO Ltd");
  add(0x14f5, "SOPAC Ltd");
  add(0x14f6, "COYOTE Technologies LLC");
  add(0x14f7, "WOLF Technology Inc");
  add(0x14f8, "AUDIOCODES Inc");
  add(0x14f9, "AG COMMUNICATIONS");
  add(0x14fa, "WANDEL & GOLTERMANN");
  add(0x14fb, "TRANSAS MARINE (UK) Ltd");
  add(0x14fc, "Quadrics Ltd");
  add(0x14fd, "JAPAN Computer Industry Inc");
  add(0x14fe, "ARCHTEK TELECOM Corp");
  add(0x14ff, "TWINHEAD INTERNATIONAL Corp");
  add(0x1500, "DELTA Electronics, Inc");
  add(0x1501, "BANKSOFT CANADA Ltd");
  add(0x1502, "MITSUBISHI ELECTRIC LOGISTICS SUPPORT Co Ltd");
  add(0x1503, "KAWASAKI LSI USA Inc");
  add(0x1504, "KAISER Electronics");
  add(0x1505, "ITA INGENIEURBURO FUR TESTAUFGABEN GmbH");
  add(0x1506, "CHAMELEON Systems Inc");
  add(0x1507, "Motorola ?? / HTEC");
  add(0x1508, "HONDA CONNECTORS/MHOTRONICS Inc");
  add(0x1509, "FIRST INTERNATIONAL Computer Inc");
  add(0x150a, "FORVUS RESEARCH Inc");
  add(0x150b, "YAMASHITA Systems Corp");
  add(0x150c, "KYOPAL CO Ltd");
  add(0x150d, "WARPSPPED Inc");
  add(0x150e, "C-PORT Corp");
  add(0x150f, "INTEC GmbH");
  add(0x1510, "BEHAVIOR TECH Computer Corp");
  add(0x1511, "CENTILLIUM Technology Corp");
  add(0x1512, "ROSUN Technologies Inc");
  add(0x1513, "Raychem");
  add(0x1514, "TFL LAN Inc");
  add(0x1515, "Advent design");
  add(0x1516, "MYSON Technology Inc");
  add(0x1517, "ECHOTEK Corp");
  add(0x1518, "Kontron Modular Computers GmbH");
  add(0x1519, "TELEFON AKTIEBOLAGET LM Ericsson");
  add(0x151a, "Globetek");
  add(0x151b, "COMBOX Ltd");
  add(0x151c, "DIGITAL AUDIO LABS Inc");
  add(0x151d, "Fujitsu Computer Products Of America");
  add(0x151e, "MATRIX Corp");
  add(0x151f, "TOPIC SEMICONDUCTOR Corp");
  add(0x1520, "CHAPLET System Inc");
  add(0x1521, "BELL Corp");
  add(0x1522, "MainPine Ltd");
  add(0x1523, "MUSIC Semiconductors");
  add(0x1524, "ENE Technology Inc");
  add(0x1525, "IMPACT Technologies");
  add(0x1526, "ISS, Inc");
  add(0x1527, "SOLECTRON");
  add(0x1528, "ACKSYS");
  add(0x1529, "AMERICAN MICROSystems Inc");
  add(0x152a, "QUICKTURN DESIGN Systems");
  add(0x152b, "FLYTECH Technology CO Ltd");
  add(0x152c, "MACRAIGOR Systems LLC");
  add(0x152d, "QUANTA Computer Inc");
  add(0x152e, "MELEC Inc");
  add(0x152f, "PHILIPS - CRYPTO");
  add(0x1530, "ACQIS Technology Inc");
  add(0x1531, "CHRYON Corp");
  add(0x1532, "ECHELON Corp");
  add(0x1533, "BALTIMORE");
  add(0x1534, "ROAD Corp");
  add(0x1535, "EVERGREEN Technologies Inc");
  add(0x1536, "ACTIS Computer");
  add(0x1537, "DATALEX COMMUNCATIONS");
  add(0x1538, "ARALION Inc");
  add(0x1539, "ATELIER INFORMATIQUES et ELECTRONIQUE ETUDES S.A.");
  add(0x153a, "ONO SOKKI");
  add(0x153b, "TERRATEC Electronic GmbH");
  add(0x153c, "ANTAL Electronic");
  add(0x153d, "FILANET Corp");
  add(0x153e, "TECHWELL Inc");
  add(0x153f, "MIPS Technologies, Inc.");
  add(0x1540, "PROVIDEO MULTIMEDIA Co Ltd");
  add(0x1541, "MACHONE Communications");
  add(0x1542, "Concurrent Computer Corporation");
  add(0x1543, "SILICON Laboratories");
  add(0x1544, "DCM DATA Systems");
  add(0x1545, "VISIONTEK");
  add(0x1546, "IOI Technology Corp");
  add(0x1547, "MITUTOYO Corp");
  add(0x1548, "JET PROPULSION Laboratory");
  add(0x1549, "INTERCONNECT Systems Solutions");
  add(0x154a, "MAX Technologies Inc");
  add(0x154b, "COMPUTEX Co Ltd");
  add(0x154c, "VISUAL Technology Inc");
  add(0x154d, "PAN INTERNATIONAL Industrial Corp");
  add(0x154e, "SERVOTEST Ltd");
  add(0x154f, "STRATABEAM Technology");
  add(0x1550, "OPEN NETWORK Co Ltd");
  add(0x1551, "SMART Electronic DEVELOPMENT GmBH");
  add(0x1552, "RACAL AIRTECH Ltd");
  add(0x1553, "CHICONY Electronics Co Ltd");
  add(0x1554, "PROLINK Microsystems Corp");
  add(0x1555, "GESYTEC GmBH");
  add(0x1556, "PLDA");
  add(0x1557, "MEDIASTAR Co Ltd");
  add(0x1558, "CLEVO/KAPOK Computer");
  add(0x1559, "SI LOGIC Ltd");
  add(0x155a, "INNOMEDIA Inc");
  add(0x155b, "PROTAC INTERNATIONAL Corp");
  add(0x155c, "Cemax-Icon Inc");
  add(0x155d, "Mac System Co Ltd");
  add(0x155e, "LP Elektronik GmbH");
  add(0x155f, "Perle Systems Ltd");
  add(0x1560, "Terayon Communications Systems");
  add(0x1561, "Viewgraphics Inc");
  add(0x1562, "Symbol Technologies");
  add(0x1563, "A-Trend Technology Co Ltd");
  add(0x1564, "Yamakatsu Electronics Industry Co Ltd");
  add(0x1565, "Biostar Microtech Int'l Corp");
  add(0x1566, "Ardent Technologies Inc");
  add(0x1567, "Jungsoft");
  add(0x1568, "DDK Electronics Inc");
  add(0x1569, "Palit Microsystems Inc.");
  add(0x156a, "Avtec Systems");
  add(0x156b, "2wire Inc");
  add(0x156c, "Vidac Electronics GmbH");
  add(0x156d, "Alpha-Top Corp");
  add(0x156e, "Alfa Inc");
  add(0x156f, "M-Systems Flash Disk Pioneers Ltd");
  add(0x1570, "Lecroy Corp");
  add(0x1571, "Contemporary Controls");
  add(0x1572, "Otis Elevator Company");
  add(0x1573, "Lattice - Vantis");
  add(0x1574, "Fairchild Semiconductor");
  add(0x1575, "Voltaire Advanced Data Security Ltd");
  add(0x1576, "Viewcast COM");
  add(0x1578, "HITT");
  add(0x1579, "Dual Technology Corp");
  add(0x157a, "Japan Elecronics Ind Inc");
  add(0x157b, "Star Multimedia Corp");
  add(0x157c, "Eurosoft (UK)");
  add(0x157d, "Gemflex Networks");
  add(0x157e, "Transition Networks");
  add(0x157f, "PX Instruments Technology Ltd");
  add(0x1580, "Primex Aerospace Co");
  add(0x1581, "SEH Computertechnik GmbH");
  add(0x1582, "Cytec Corp");
  add(0x1583, "Inet Technologies Inc");
  add(0x1584, "Uniwill Computer Corp");
  add(0x1585, "Logitron");
  add(0x1586, "Lancast Inc");
  add(0x1587, "Konica Corp");
  add(0x1588, "Solidum Systems Corp");
  add(0x1589, "Atlantek Microsystems Pty Ltd");
  add(0x158a, "Digalog Systems Inc");
  add(0x158b, "Allied Data Technologies");
  add(0x158c, "Hitachi Semiconductor & Devices Sales Co Ltd");
  add(0x158d, "Point Multimedia Systems");
  add(0x158e, "Lara Technology Inc");
  add(0x158f, "Ditect Coop");
  add(0x1590, "Hewlett-Packard Company");
  add(0x1591, "ARN");
  add(0x1592, "Syba Tech Ltd");
  add(0x1593, "Bops Inc");
  add(0x1594, "Netgame Ltd");
  add(0x1595, "Diva Systems Corp");
  add(0x1596, "Folsom Research Inc");
  add(0x1597, "Memec Design Services");
  add(0x1598, "Granite Microsystems");
  add(0x1599, "Delta Electronics Inc");
  add(0x159a, "General Instrument");
  add(0x159b, "Faraday Technology Corp");
  add(0x159c, "Stratus Computer Systems");
  add(0x159d, "Ningbo Harrison Electronics Co Ltd");
  add(0x159e, "A-Max Technology Co Ltd");
  add(0x159f, "Galea Network Security");
  add(0x15a0, "Compumaster SRL");
  add(0x15a1, "Geocast Network Systems");
  add(0x15a2, "Catalyst Enterprises Inc");
  add(0x15a3, "Italtel");
  add(0x15a4, "X-Net OY");
  add(0x15a5, "Toyota Macs Inc");
  add(0x15a6, "Sunlight Ultrasound Technologies Ltd");
  add(0x15a7, "SSE Telecom Inc");
  add(0x15a8, "Shanghai Communications Technologies Center");
  add(0x15aa, "Moreton Bay");
  add(0x15ab, "Bluesteel Networks Inc");
  add(0x15ac, "North Atlantic Instruments");
  add(0x15ad, "VMware");
  add(0x15ae, "Amersham Pharmacia Biotech");
  add(0x15b0, "Zoltrix International Ltd");
  add(0x15b1, "Source Technology Inc");
  add(0x15b2, "Mosaid Technologies Inc");
  add(0x15b3, "Mellanox Technologies");
  add(0x15b4, "CCI/TRIAD");
  add(0x15b5, "Cimetrics Inc");
  add(0x15b6, "Texas Memory Systems Inc");
  add(0x15b7, "Sandisk Corp");
  add(0x15b8, "ADDI-DATA GmbH");
  add(0x15b9, "Maestro Digital Communications");
  add(0x15ba, "Impacct Technology Corp");
  add(0x15bb, "Portwell Inc");
  add(0x15bc, "Agilent Technologies");
  add(0x15bd, "DFI Inc");
  add(0x15be, "Sola Electronics");
  add(0x15bf, "High Tech Computer Corp (HTC)");
  add(0x15c0, "BVM Ltd");
  add(0x15c1, "Quantel");
  add(0x15c2, "Newer Technology Inc");
  add(0x15c3, "Taiwan Mycomp Co Ltd");
  add(0x15c4, "EVSX Inc");
  add(0x15c5, "Procomp Informatics Ltd");
  add(0x15c6, "Technical University of Budapest");
  add(0x15c7, "Tateyama System Laboratory Co Ltd");
  add(0x15c8, "Penta Media Co Ltd");
  add(0x15c9, "Serome Technology Inc");
  add(0x15ca, "Bitboys OY");
  add(0x15cb, "AG Electronics Ltd");
  add(0x15cc, "Hotrail Inc");
  add(0x15cd, "Dreamtech Co Ltd");
  add(0x15ce, "Genrad Inc");
  add(0x15cf, "Hilscher GmbH");
  add(0x15d1, "Infineon Technologies AG");
  add(0x15d2, "FIC (First International Computer Inc)");
  add(0x15d3, "NDS Technologies Israel Ltd");
  add(0x15d4, "Iwill Corp");
  add(0x15d5, "Tatung Co");
  add(0x15d6, "Entridia Corp");
  add(0x15d7, "Rockwell-Collins Inc");
  add(0x15d8, "Cybernetics Technology Co Ltd");
  add(0x15d9, "Super Micro Computer Inc");
  add(0x15da, "Cyberfirm Inc");
  add(0x15db, "Applied Computing Systems Inc");
  add(0x15dc, "Litronic Inc");
  add(0x15dd, "Sigmatel Inc");
  add(0x15de, "Malleable Technologies Inc");
  add(0x15df, "Infinilink Corp");
  add(0x15e0, "Cacheflow Inc");
  add(0x15e1, "Voice Technologies Group Inc");
  add(0x15e2, "Quicknet Technologies Inc");
  add(0x15e3, "Networth Technologies Inc");
  add(0x15e4, "VSN Systemen BV");
  add(0x15e5, "Valley technologies Inc");
  add(0x15e6, "Agere Inc");
  add(0x15e7, "Get Engineering Corp");
  add(0x15e8, "National Datacomm Corp");
  add(0x15e9, "Pacific Digital Corp");
  add(0x15ea, "Tokyo Denshi Sekei K.K.");
  add(0x15eb, "DResearch Digital Media Systems GmbH");
  add(0x15ec, "Beckhoff GmbH");
  add(0x15ed, "Macrolink Inc");
  add(0x15ee, "In Win Development Inc");
  add(0x15ef, "Intelligent Paradigm Inc");
  add(0x15f0, "B-Tree Systems Inc");
  add(0x15f1, "Times N Systems Inc");
  add(0x15f2, "Diagnostic Instruments Inc");
  add(0x15f3, "Digitmedia Corp");
  add(0x15f4, "Valuesoft");
  add(0x15f5, "Power Micro Research");
  add(0x15f6, "Extreme Packet Device Inc");
  add(0x15f7, "Banctec");
  add(0x15f8, "Koga Electronics Co");
  add(0x15f9, "Zenith Electronics Corp");
  add(0x15fa, "J.P. Axzam Corp");
  add(0x15fb, "Zilog Inc");
  add(0x15fc, "Techsan Electronics Co Ltd");
  add(0x15fd, "N-CUBED.NET");
  add(0x15fe, "Kinpo Electronics Inc");
  add(0x15ff, "Fastpoint Technologies Inc");
  add(0x1600, "Northrop Grumman - Canada Ltd");
  add(0x1601, "Tenta Technology");
  add(0x1602, "Prosys-tec Inc");
  add(0x1603, "Nokia Wireless Communications");
  add(0x1604, "Central System Research Co Ltd");
  add(0x1605, "Pairgain Technologies");
  add(0x1606, "Europop AG");
  add(0x1607, "Lava Semiconductor Manufacturing Inc");
  add(0x1608, "Automated Wagering International");
  add(0x1609, "Scimetric Instruments Inc");
  add(0x160a, "Kollmorgen Servotronix");
  add(0x1612, "Telesynergy Research Inc.");
  add(0x1616, "Iotech Inc.");
  add(0x1618, "Stone Ridge Technology");
  add(0x1619, "FarSite Communications Ltd");
  add(0x161f, "Rioworks");
  add(0x1621, "Lynx Studio Technology Inc");
  add(0x1626, "TDK Semiconductor Corp.");
  add(0x1629, "Kongsberg Spacetec AS");
  add(0x162d, "Reprosoft Co Ltd");
  add(0x162f, "Rohde & Schwarz GMBH & Co KG");
  add(0x1631, "Packard Bell B.V.");
  add(0x1638, "Standard Microsystems Corp [SMC]");
  add(0x163b, "Glotrex Co Ltd");
  add(0x163c, "Smart Link Ltd.");
  add(0x1641, "MKNet Corp.");
  add(0x1642, "Bitland(ShenZhen) Information Technology Co., Ltd.");
  add(0x164f, "Datavoice (Pty) Ltd.");
  add(0x1657, "Brocade Communications Systems, Inc.");
  add(0x1658, "Med Associates Inc.");
  add(0x165a, "Epix Inc");
  add(0x165c, "Kondo Kagaku");
  add(0x165d, "Hsing Tech. Enterprise Co., Ltd.");
  add(0x165f, "Linux Media Labs, LLC");
  add(0x1661, "Worldspace Corp.");
  add(0x1668, "Actiontec Electronics Inc");
  add(0x166d, "Broadcom Corporation");
  add(0x1676, "Emachines Inc.");
  add(0x1677, "Bernecker + Rainer");
  add(0x1678, "NetEffect");
  add(0x1679, "Tokyo Electron Device Ltd.");
  add(0x167b, "ZyDAS Technology Corp.");
  add(0x167d, "Samsung Electro-Mechanics Co., Ltd.");
  add(0x167e, "ONNTO Corp.");
  add(0x167f, "iba AG");
  add(0x1681, "Hercules");
  add(0x1682, "XFX Pine Group Inc.");
  add(0x1688, "CastleNet Technology Inc.");
  add(0x168c, "Qualcomm Atheros");
  add(0x1693, "FERMA");
  add(0x1695, "EPoX Computer Co., Ltd.");
  add(0x169c, "Netcell Corporation");
  add(0x169d, "Club-3D VB (Wrong ID)");
  add(0x16a5, "Tekram Technology Co.,Ltd.");
  add(0x16ab, "Global Sun Technology Inc");
  add(0x16ae, "SafeNet Inc");
  add(0x16af, "SparkLAN Communications, Inc.");
  add(0x16b4, "Aspex Semiconductor Ltd");
  add(0x16b8, "Sonnet Technologies, Inc.");
  add(0x16be, "Creatix Polymedia GmbH");
  add(0x16c6, "Micrel-Kendin");
  add(0x16c8, "Octasic Inc.");
  add(0x16c9, "EONIC B.V. The Netherlands");
  add(0x16ca, "CENATEK Inc");
  add(0x16cd, "Advantech Co. Ltd");
  add(0x16ce, "Roland Corp.");
  add(0x16d5, "Acromag, Inc.");
  add(0x16da, "Advantech Co., Ltd.");
  add(0x16df, "PIKA Technologies Inc.");
  add(0x16e2, "Geotest-MTS");
  add(0x16e3, "European Space Agency");
  add(0x16e5, "Intellon Corp.");
  add(0x16ec, "U.S. Robotics");
  add(0x16ed, "Sycron N. V.");
  add(0x16f3, "Jetway Information Co., Ltd.");
  add(0x16f4, "Vweb Corp");
  add(0x16f6, "VideoTele.com, Inc.");
  add(0x1702, "Internet Machines Corporation (IMC)");
  add(0x1705, "Digital First, Inc.");
  add(0x170b, "NetOctave");
  add(0x170c, "YottaYotta Inc.");
  add(0x1710, "Pelago Nutworks");
  add(0x1712, "NICE Systems Inc.");
  add(0x1719, "EZChip Technologies");
  add(0x1725, "Vitesse Semiconductor");
  add(0x172a, "Accelerated Encryption");
  add(0x1734, "Fujitsu Technology Solutions");
  add(0x1735, "Aten International Co. Ltd.");
  add(0x1737, "Linksys");
  add(0x173b, "Altima (nee Broadcom)");
  add(0x1743, "Peppercon AG");
  add(0x1745, "ViXS Systems, Inc.");
  add(0x1749, "RLX Technologies");
  add(0x174b, "PC Partner Limited / Sapphire Technology");
  add(0x174d, "WellX Telecom SA");
  add(0x1753, "TeraRecon, Inc.");
  add(0x1755, "Alchemy Semiconductor Inc.");
  add(0x175c, "AudioScience Inc");
  add(0x175e, "Sanera Systems, Inc.");
  add(0x1760, "TEDIA spol. s r. o.");
  add(0x1771, "InnoVISION Multimedia Ltd.");
  add(0x1775, "GE Intelligent Platforms");
  add(0x177d, "Cavium Networks");
  add(0x1787, "Hightech Information System Ltd.");
  add(0x1789, "Ennyah Technologies Corp.");
  add(0x1796, "Research Centre Juelich");
  add(0x1797, "Techwell Inc.");
  add(0x1799, "Belkin");
  add(0x179a, "id Quantique");
  add(0x179c, "Data Patterns");
  add(0x17a0, "Genesys Logic, Inc");
  add(0x17a1, "Tascorp");
  add(0x17aa, "Lenovo");
  add(0x17ab, "Phillips Components");
  add(0x17af, "Hightech Information System Ltd.");
  add(0x17b3, "Hawking Technologies");
  add(0x17b4, "Indra Networks, Inc.");
  add(0x17c0, "Wistron Corp.");
  add(0x17c2, "Newisys, Inc.");
  add(0x17cb, "Airgo Networks, Inc.");
  add(0x17cc, "NetChip Technology, Inc");
  add(0x17cf, "Z-Com, Inc.");
  add(0x17d3, "Areca Technology Corp.");
  add(0x17d5, "Exar Corp.");
  add(0x17db, "Cray Inc");
  add(0x17de, "KWorld Computer Co. Ltd.");
  add(0x17e4, "Sectra AB");
  add(0x17e6, "Entropic Communications Inc.");
  add(0x17e9, "DH electronics GmbH / Sabrent");
  add(0x17ee, "Connect Components Ltd");
  add(0x17f2, "Albatron Corp.");
  add(0x17f3, "RDC Semiconductor, Inc.");
  add(0x17f7, "Topdek Semiconductor Inc.");
  add(0x17f9, "Gemtek Technology Co., Ltd");
  add(0x17fc, "IOGEAR, Inc.");
  add(0x17fe, "InProComm Inc.");
  add(0x17ff, "Benq Corporation");
  add(0x1803, "ProdaSafe GmbH");
  add(0x1805, "Euresys S.A.");
  add(0x1809, "Lumanate, Inc.");
  add(0x1813, "Ambient Technologies Inc");
  add(0x1814, "Ralink corp.");
  add(0x1815, "Devolo AG");
  add(0x1820, "InfiniCon Systems Inc.");
  add(0x1822, "Twinhan Technology Co. Ltd");
  add(0x182d, "SiteCom Europe BV");
  add(0x182e, "Raza Microelectronics, Inc.");
  add(0x182f, "Broadcom");
  add(0x1830, "Credence Systems Corporation");
  add(0x183b, "MikroM GmbH");
  add(0x1846, "Alcatel-Lucent");
  add(0x1849, "ASRock Incorporation");
  add(0x184a, "Thales Computers");
  add(0x1851, "Microtune, Inc.");
  add(0x1852, "Anritsu Corp.");
  add(0x1853, "SMSC Automotive Infotainment System Group");
  add(0x1854, "LG Electronics, Inc.");
  add(0x185b, "Compro Technology, Inc.");
  add(0x185f, "Wistron NeWeb Corp.");
  add(0x1864, "SilverBack");
  add(0x1867, "Topspin Communications");
  add(0x186c, "Humusoft, s.r.o.");
  add(0x186f, "WiNRADiO Communications");
  add(0x1876, "L-3 Communications");
  add(0x187e, "ZyXEL Communications Corporation");
  add(0x1885, "Avvida Systems Inc.");
  add(0x1888, "Varisys Ltd");
  add(0x188a, "Ample Communications, Inc");
  add(0x1890, "Egenera, Inc.");
  add(0x1894, "KNC One");
  add(0x1896, "B&B Electronics Manufacturing Company, Inc.");
  add(0x1897, "AMtek");
  add(0x18a1, "Astute Networks Inc.");
  add(0x18a2, "Stretch Inc.");
  add(0x18a3, "AT&T");
  add(0x18ac, "DViCO Corporation");
  add(0x18b8, "Ammasso");
  add(0x18bc, "GeCube Technologies, Inc.");
  add(0x18c3, "Micronas Semiconductor Holding AG");
  add(0x18c8, "Cray Inc");
  add(0x18c9, "ARVOO Engineering BV");
  add(0x18ca, "XGI Technology Inc. (eXtreme Graphics Innovation)");
  add(0x18d2, "Sitecom Europe BV (Wrong ID)");
  add(0x18d8, "Dialogue Technology Corp.");
  add(0x18dd, "Artimi Inc");
  add(0x18df, "LeWiz Communications");
  add(0x18e6, "MPL AG");
  add(0x18eb, "Advance Multimedia Internet Technology, Inc.");
  add(0x18ec, "Cesnet, z.s.p.o.");
  add(0x18ee, "Chenming Mold Ind. Corp.");
  add(0x18f1, "Spectrum GmbH");
  add(0x18f4, "Napatech A/S");
  add(0x18f6, "NextIO");
  add(0x18f7, "Commtech, Inc.");
  add(0x18fb, "Resilience Corporation");
  add(0x1904, "Hangzhou Silan Microelectronics Co., Ltd.");
  add(0x1905, "Micronas USA, Inc.");
  add(0x1910, "Seaway Networks");
  add(0x1912, "Renesas Technology Corp.");
  add(0x1919, "Soltek Computer Inc.");
  add(0x1923, "Sangoma Technologies Corp.");
  add(0x1924, "Solarflare Communications");
  add(0x192a, "BiTMICRO Networks Inc.");
  add(0x192e, "TransDimension");
  add(0x1931, "Option N.V.");
  add(0x1932, "DiBcom");
  add(0x193c, "MAXIM Integrated Products");
  add(0x193f, "Comtech AHA Corp.");
  add(0x1942, "ClearSpeed Technology plc");
  add(0x1947, "C-guys, Inc.");
  add(0x1948, "Alpha Networks Inc.");
  add(0x194a, "DapTechnology B.V.");
  add(0x1954, "One Stop Systems, Inc.");
  add(0x1957, "Freescale Semiconductor Inc");
  add(0x1958, "Faster Technology, LLC.");
  add(0x1959, "PA Semi, Inc");
  add(0x1966, "Orad Hi-Tec Systems");
  add(0x1969, "Qualcomm Atheros");
  add(0x196a, "Sensory Networks Inc.");
  add(0x196d, "Club-3D BV");
  add(0x1971, "AGEIA Technologies, Inc.");
  add(0x1974, "Eberspaecher Electronics");
  add(0x1976, "TRENDnet");
  add(0x1977, "Parsec");
  add(0x197b, "JMicron Technology Corp.");
  add(0x1982, "Distant Early Warning Communications Inc");
  add(0x1989, "Montilio Inc.");
  add(0x198a, "Nallatech Ltd.");
  add(0x1993, "Innominate Security Technologies AG");
  add(0x1999, "A-Logics");
  add(0x199a, "Pulse-LINK, Inc.");
  add(0x199d, "Xsigo Systems");
  add(0x199f, "Auvitek");
  add(0x19a2, "Emulex Corporation");
  add(0x19a8, "DAQDATA GmbH");
  add(0x19ac, "Kasten Chase Applied Research");
  add(0x19ae, "Progeny Systems Corporation");
  add(0x19b6, "Mikrotik");
  add(0x19c1, "Exegy Inc.");
  add(0x19d1, "Motorola Expedience");
  add(0x19d4, "Quixant Limited");
  add(0x19da, "ZOTAC International (MCO) Ltd.");
  add(0x19de, "Pico Computing");
  add(0x19e2, "Vector Informatik GmbH");
  add(0x19e3, "DDRdrive LLC");
  add(0x19e7, "NET (Network Equipment Technologies)");
  add(0x19ee, "Netronome Systems, Inc.");
  add(0x19f1, "BFG Tech");
  add(0x19ff, "Eclipse Electronic Systems, Inc.");
  add(0x1a03, "ASPEED Technology, Inc.");
  add(0x1a07, "Kvaser AB");
  add(0x1a08, "Sierra semiconductor");
  add(0x1a0e, "DekTec Digital Video B.V.");
  add(0x1a17, "Force10 Networks, Inc.");
  add(0x1a1d, "GFaI e.V.");
  add(0x1a1e, "3Leaf Systems, Inc.");
  add(0x1a22, "Ambric Inc.");
  add(0x1a29, "Fortinet, Inc.");
  add(0x1a2b, "Ascom AG");
  add(0x1a32, "Quanta Microsystems, Inc");
  add(0x1a3b, "AzureWave");
  add(0x1a41, "Tilera Corp.");
  add(0x1a4a, "SLAC National Accelerator Lab PPA-REG");
  add(0x1a51, "Hectronic AB");
  add(0x1a55, "Rohde & Schwarz DVS GmbH");
  add(0x1a56, "Bigfoot Networks, Inc.");
  add(0x1a57, "Highly Reliable Systems");
  add(0x1a58, "Razer USA Ltd.");
  add(0x1a5d, "Celoxica");
  add(0x1a5e, "Aprius Inc.");
  add(0x1a5f, "System TALKS Inc.");
  add(0x1a68, "VirtenSys Limited");
  add(0x1a71, "XenSource, Inc.");
  add(0x1a73, "Violin Memory, Inc");
  add(0x1a76, "Wavesat");
  add(0x1a77, "Lightfleet Corporation");
  add(0x1a78, "Virident Systems Inc.");
  add(0x1a84, "Commex Technologies");
  add(0x1a88, "MEN Mikro Elektronik");
  add(0x1a8c, "Verigy Pte. Ltd.");
  add(0x1a8e, "DRS Technologies");
  add(0x1aa8, "Ciprico, Inc.");
  add(0x1aae, "Global Velocity, Inc.");
  add(0x1ab6, "CalDigit, Inc.");
  add(0x1ab8, "Parallels, Inc.");
  add(0x1ab9, "Espia Srl");
  add(0x1acc, "Point of View BV");
  add(0x1ad7, "Spectracom Corporation");
  add(0x1ade, "Spin Master Ltd.");
  add(0x1ae0, "Google, Inc.");
  add(0x1ae7, "First Wise Media GmbH");
  add(0x1ae8, "Silicon Software GmbH");
  add(0x1ae9, "Wilocity Ltd.");
  add(0x1aec, "Wolfson Microelectronics");
  add(0x1aed, "Fusion-io");
  add(0x1aee, "Caustic Graphics Inc.");
  add(0x1af4, "Red Hat, Inc");
  add(0x1af5, "Netezza Corp.");
  add(0x1afa, "J & W Electronics Co., Ltd.");
  add(0x1b03, "Magnum Semiconductor, Inc,");
  add(0x1b08, "MSC Vertriebs GmbH");
  add(0x1b0a, "Pegatron");
  add(0x1b13, "Jaton Corp");
  add(0x1b1a, "K&F Computing Research Co.");
  add(0x1b21, "ASMedia Technology Inc.");
  add(0x1b2c, "Opal-RT Technologies Inc.");
  add(0x1b36, "Red Hat, Inc.");
  add(0x1b37, "Signal Processing Devices Sweden AB");
  add(0x1b3a, "Westar Display Technologies");
  add(0x1b3e, "Teradata Corp.");
  add(0x1b40, "Schooner Information Technology, Inc.");
  add(0x1b47, "Numascale AS");
  add(0x1b4b, "Marvell Technology Group Ltd.");
  add(0x1b55, "NetUP Inc.");
  add(0x1b6f, "Etron Technology, Inc.");
  add(0x1b73, "Fresco Logic");
  add(0x1b74, "OpenVox Communication Co. Ltd.");
  add(0x1b85, "OCZ Technology Group, Inc.");
  add(0x1b96, "Western Digital");
  add(0x1b9a, "XAVi Technologies Corp.");
  add(0x1bad, "ReFLEX CES");
  add(0x1bb0, "SimpliVity Corporation");
  add(0x1bb3, "Bluecherry");
  add(0x1bb5, "Quantenna Communications, Inc.");
  add(0x1bbf, "Maxeler Technologies Ltd.");
  add(0x1bf4, "VTI Instruments Corporation");
  add(0x1bfd, "EeeTOP");
  add(0x1c1c, "Symphony");
  add(0x1c2c, "Fiberblaze");
  add(0x1c32, "Highland Technology, Inc.");
  add(0x1c33, "Daktronics, Inc");
  add(0x1c39, "Thomson Video Networks");
  add(0x1c3b, "Accensus, LLC");
  add(0x1c44, "Enmotus Inc");
  add(0x1c7f, "Elektrobit Austria GmbH");
  add(0x1c8a, "TSF5 Corporation");
  add(0x1cb1, "Collion UG & Co.KG");
  add(0x1cc5, "Embedded Intelligence, Inc.");
  add(0x1ce4, "Exablaze");
  add(0x1cf7, "Subspace Dynamics");
  add(0x1d44, "DPT");
  add(0x1d5c, "Fantasia Trading LLC");
  add(0x1de1, "Tekram Technology Co.,Ltd.");
  add(0x1fc0, "Ascom (Finland) Oy");
  add(0x1fc1, "QLogic, Corp.");
  add(0x1fc9, "Tehuti Networks Ltd.");
  add(0x1fce, "Cognio Inc.");
  add(0x1fd4, "SUNIX Co., Ltd.");
  add(0x2000, "Smart Link Ltd.");
  add(0x2001, "Temporal Research Ltd");
  add(0x2003, "Smart Link Ltd.");
  add(0x2004, "Smart Link Ltd.");
  add(0x20f4, "TRENDnet");
  add(0x2116, "ZyDAS Technology Corp.");
  add(0x21c3, "21st Century Computer Corp.");
  add(0x2304, "Colorgraphic Communications Corp.");
  add(0x2348, "Racore");
  add(0x2646, "Kingston Technologies");
  add(0x270b, "Xantel Corporation");
  add(0x270f, "Chaintech Computer Co. Ltd");
  add(0x2711, "AVID Technology Inc.");
  add(0x2955, "Connectix Virtual PC");
  add(0x2a15, "3D Vision");
  add(0x3000, "Hansol Electronics Inc.");
  add(0x3142, "Post Impression Systems.");
  add(0x31ab, "Zonet");
  add(0x3388, "Hint Corp");
  add(0x3411, "Quantum Designs (H.K.) Inc");
  add(0x3442, "Bihl+Wiedemann GmbH");
  add(0x3475, "Arastra Inc.");
  add(0x3513, "ARCOM Control Systems Ltd");
  add(0x37d9, "ITD Firm ltd.");
  add(0x3842, "eVga.com. Corp.");
  add(0x38ef, "4Links");
  add(0x3d3d, "3DLabs");
  add(0x4005, "Avance Logic Inc.");
  add(0x4033, "Addtron Technology Co, Inc.");
  add(0x4040, "NetXen Incorporated");
  add(0x4143, "Digital Equipment Corp");
  add(0x4144, "Alpha Data");
  add(0x4150, "ONA Electroerosion");
  add(0x415a, "Auzentech, Inc.");
  add(0x416c, "Aladdin Knowledge Systems");
  add(0x4321, "Tata Power Strategic Electronics Division");
  add(0x4348, "wch.cn");
  add(0x434e, "CAST Navigation LLC");
  add(0x4444, "Internext Compression Inc");
  add(0x4468, "Bridgeport machines");
  add(0x4594, "Cogetec Informatique Inc");
  add(0x45fb, "Baldor Electric Company");
  add(0x4624, "Budker Institute of Nuclear Physics");
  add(0x4680, "Umax Computer Corp");
  add(0x4843, "Hercules Computer Technology Inc");
  add(0x4916, "RedCreek Communications Inc");
  add(0x4943, "Growth Networks");
  add(0x494f, "ACCES I/O Products, Inc.");
  add(0x4978, "Axil Computer Inc");
  add(0x4a14, "NetVin");
  add(0x4b10, "Buslogic Inc.");
  add(0x4c48, "LUNG HWA Electronics");
  add(0x4c53, "SBS Technologies");
  add(0x4ca1, "Seanix Technology Inc");
  add(0x4d51, "MediaQ Inc.");
  add(0x4d54, "Microtechnica Co Ltd");
  add(0x4d56, "MATRIX VISION GmbH");
  add(0x4ddc, "ILC Data Device Corp");
  add(0x5045, "University of Toronto");
  add(0x5046, "GemTek Technology Corporation");
  add(0x5053, "Voyetra Technologies");
  add(0x50b2, "TerraTec Electronic GmbH");
  add(0x5136, "S S Technologies");
  add(0x5143, "Qualcomm Inc");
  add(0x5145, "Ensoniq (Old)");
  add(0x5168, "Animation Technologies Inc.");
  add(0x5301, "Alliance Semiconductor Corp.");
  add(0x5333, "S3 Graphics Ltd.");
  add(0x5431, "AuzenTech, Inc.");
  add(0x544c, "Teralogic Inc");
  add(0x5452, "SCANLAB AG");
  add(0x5455, "Technische University Berlin");
  add(0x5456, "GoTView");
  add(0x5519, "Cnet Technologies, Inc.");
  add(0x5544, "Dunord Technologies");
  add(0x5555, "Genroco, Inc");
  add(0x5646, "Vector Fabrics BV");
  add(0x5654, "VoiceTronix Pty Ltd");
  add(0x5700, "Netpower");
  add(0x584d, "AuzenTech Co., Ltd.");
  add(0x5851, "Exacq Technologies");
  add(0x5853, "XenSource, Inc.");
  add(0x5854, "GoTView");
  add(0x5ace, "Beholder International Ltd.");
  add(0x631c, "SmartInfra Ltd");
  add(0x6356, "UltraStor");
  add(0x6374, "c't Magazin fuer Computertechnik");
  add(0x6409, "Logitec Corp.");
  add(0x6549, "Teradici Corp.");
  add(0x6666, "Decision Computer International Co.");
  add(0x6688, "Zycoo Co., Ltd");
  add(0x6900, "Red Hat, Inc.");
  add(0x7063, "pcHDTV");
  add(0x7284, "HT OMEGA Inc.");
  add(0x7604, "O.N. Electronic Co Ltd.");
  add(0x7bde, "MIDAC Corporation");
  add(0x7fed, "PowerTV");
  add(0x8008, "Quancom Electronic GmbH");
  add(0x807d, "Asustek Computer, Inc.");
  add(0x8080, "Xirlink, Inc");
  add(0x8086, "Intel Corporation");
  add(0x8087, "Intel");
  add(0x80ee, "InnoTek Systemberatung GmbH");
  add(0x8322, "Sodick America Corp.");
  add(0x8384, "SigmaTel");
  add(0x8401, "TRENDware International Inc.");
  add(0x8686, "ScaleMP");
  add(0x8800, "Trigem Computer Inc.");
  add(0x8866, "T-Square Design Inc.");
  add(0x8888, "Silicon Magic");
  add(0x8912, "TRX");
  add(0x8c4a, "Winbond");
  add(0x8e0e, "Computone Corporation");
  add(0x8e2e, "KTI");
  add(0x9004, "Adaptec");
  add(0x9005, "Adaptec");
  add(0x907f, "Atronics");
  add(0x919a, "Gigapixel Corp");
  add(0x9412, "Holtek");
  add(0x9618, "JusonTech Corporation");
  add(0x9699, "Omni Media Technology Inc");
  add(0x9710, "MosChip Semiconductor Technology Ltd.");
  add(0x9902, "Stargen Inc.");
  add(0xa0a0, "AOPEN Inc.");
  add(0xa0f1, "UNISYS Corporation");
  add(0xa200, "NEC Corporation");
  add(0xa259, "Hewlett Packard");
  add(0xa25b, "Hewlett Packard GmbH PL24-MKT");
  add(0xa304, "Sony");
  add(0xa727, "3Com Corporation");
  add(0xaa42, "Scitex Digital Video");
  add(0xaa55, "Ncomputing X300 PCI-Engine");
  add(0xaaaa, "Adnaco Technology Inc.");
  add(0xabcd, "Vadatech Inc.");
  add(0xac1e, "Digital Receiver Technology Inc");
  add(0xac3d, "Actuality Systems");
  add(0xad00, "Alta Data Technologies LLC");
  add(0xaecb, "Adrienne Electronics Corporation");
  add(0xaffe, "Sirrix AG security technologies");
  add(0xb100, "OpenVox Communication Co. Ltd.");
  add(0xb10b, "Uakron PCI Project");
  add(0xb1b3, "Shiva Europe Limited");
  add(0xb1d9, "ATCOM Technology co., LTD.");
  add(0xbd11, "Pinnacle Systems, Inc. (Wrong ID)");
  add(0xbdbd, "Blackmagic Design");
  add(0xc001, "TSI Telsys");
  add(0xc0a9, "Micron/Crucial Technology");
  add(0xc0de, "Motorola");
  add(0xc0fe, "Motion Engineering, Inc.");
  add(0xca50, "Varian Australia Pty Ltd");
  add(0xcace, "CACE Technologies, Inc.");
  add(0xcaed, "Canny Edge");
  add(0xcafe, "Chrysalis-ITS");
  add(0xcccc, "Catapult Communications");
  add(0xccec, "Curtiss-Wright Controls Embedded Computing");
  add(0xcddd, "Tyzx, Inc.");
  add(0xceba, "KEBA AG");
  add(0xd161, "Digium, Inc.");
  add(0xd4d4, "Dy4 Systems Inc");
  add(0xd531, "I+ME ACTIA GmbH");
  add(0xd84d, "Exsys");
  add(0xdada, "Datapath Limited");
  add(0xdb10, "Diablo Technologies");
  add(0xdcba, "Dynamic Engineering");
  add(0xdd01, "Digital Devices GmbH");
  add(0xdead, "Indigita Corporation");
  add(0xdeaf, "Middle Digital Inc.");
  add(0xdeda, "SoftHard Technology Ltd.");
  add(0xe000, "Winbond");
  add(0xe159, "Tiger Jet Network Inc.");
  add(0xe1c5, "Elcus");
  add(0xe4bf, "EKF Elektronik GmbH");
  add(0xe55e, "Essence Technology, Inc.");
  add(0xea01, "Eagle Technology");
  add(0xea60, "RME");
  add(0xeabb, "Aashima Technology B.V.");
  add(0xeace, "Endace Measurement Systems, Ltd");
  add(0xec80, "Belkin Corporation");
  add(0xecc0, "Echo Digital Audio Corporation");
  add(0xedd8, "ARK Logic Inc");
  add(0xf043, "ASUSTeK Computer Inc. (Wrong ID)");
  add(0xf05b, "Foxconn International, Inc. (Wrong ID)");
  add(0xf1d0, "AJA Video");
  add(0xf5f5, "F5 Networks, Inc.");
  add(0xf849, "ASRock Incorporation (Wrong ID)");
  add(0xfa57, "Interagon AS");
  add(0xfab7, "Fabric7 Systems, Inc.");
  add(0xfebd, "Ultraview Corp.");
  add(0xfeda, "Broadcom Inc");
  add(0xfede, "Fedetec Inc.");
  add(0xfffd, "XenSource, Inc.");
  add(0xfffe, "VMWare Inc (temporary ID)");
  add(0xffff, "Illegal Vendor ID");
}
